Package: latte
Type: Package
Title: Interface to 'LattE' and '4ti2'
Version: 0.2.2
Authors@R: c(person("David", "Kahle", email = "david@kahle.io", role = c("aut", "cph", "cre"), comment = c(ORCID = "0000-0002-9999-1558")),
	person("Luis", "Garcia",email = "lgarcia@shsu.edu", role = c("aut", "cph")),
	person("Ruriko", "Yoshida", email = "ryoshida@nps.edu", role = c("aut", "cph")))
Maintainer: David Kahle <david@kahle.io>
Description: Back-end connections to 'LattE' (<https://www.math.ucdavis.edu/~latte/>) 
	for counting lattice points and integration inside convex polytopes and 
	'4ti2' (<http://www.4ti2.de/>) for algebraic, geometric, and combinatorial 
	problems on linear spaces and front-end tools facilitating their use in the 
	'R' ecosystem.
License: GPL-2
URL: https://github.com/dkahle/latte
BugReports: https://github.com/dkahle/latte/issues
SystemRequirements: LattE <https://www.math.ucdavis.edu/~latte/>, 4ti2
        <http://www.4ti2.de/>
Imports: magrittr, stringr, mpoly, ggplot2, memoise, dplyr, usethis,
        glue
Suggests: knitr, rmarkdown
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-06-09 05:07:18 UTC; david_kahle
Author: David Kahle [aut, cph, cre] (ORCID:
    <https://orcid.org/0000-0002-9999-1558>),
  Luis Garcia [aut, cph],
  Ruriko Yoshida [aut, cph]
Repository: CRAN
Date/Publication: 2025-06-09 05:20:02 UTC
Built: R 4.6.0; ; 2026-04-29 20:46:05 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,tableau)
export(edit_r_environ)
export(fgraver)
export(fgroebner)
export(fhilbert)
export(fmarkov)
export(format_latte)
export(fzbasis)
export(genmodel)
export(get_4ti2_path)
export(get_latte_path)
export(graver)
export(groebner)
export(has_4ti2)
export(has_latte)
export(hilbert)
export(kprod)
export(latte_count)
export(latte_fcount)
export(latte_max)
export(latte_min)
export(markov)
export(missing_4ti2_stop)
export(missing_latte_stop)
export(ones)
export(plot_matrix)
export(ppi)
export(qsolve)
export(read.latte)
export(read_latte)
export(set_4ti2_path)
export(set_latte_path)
export(tab2vec)
export(tableau)
export(vec2tab)
export(write.latte)
export(write_latte)
export(zbasis)
export(zsolve)
import(mpoly)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(glue,glue)
importFrom(magrittr,"%>%")
importFrom(memoise,memoise)
importFrom(stats,runif)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(stringr,str_trim)
importFrom(stringr,str_which)
importFrom(usethis,edit_r_environ)
importFrom(utils,download.file)

#' Compute the primitive partition identities
#'
#' ppi runs 4ti2's ppi program to compute the primitive partition identities,
#' that is, the Graver basis of 1:N.
#'
#' @param N A postive integer > 2
#' @param dir Directory to place the files in, without an ending /
#' @param quiet If FALSE, messages the 4ti2 output
#' @param shell Messages the shell code used to do the computation
#' @param ... Additional arguments to pass to the function
#' @return A matrix containing the basis as its columns (for easy addition to
#'   tables)
#' @seealso [graver()]
#' @export
#' @examples
#'
#' if (has_4ti2()) {
#'
#' ppi(3)
#' t(ppi(3)) %*% 1:3
#' plot_matrix(ppi(3))
#'
#' graver(t(1:3))
#' plot_matrix(graver(t(1:3)))
#'
#' ppi(5, quiet = FALSE, shell = TRUE)
#'
#' }
#'
#' 
ppi <- function(N, dir = tempdir(), quiet = TRUE, shell = FALSE, ...){

  
  if (!has_4ti2()) missing_4ti2_stop()
  
  ## arg checking
  stopifnot(is.wholenumber(N) && N > 2)

  ## compute other args
  opts <- as.list(match.call(expand.dots = FALSE))[["..."]]
  if(is.null(opts)){
    opts <- ""
  } else {
    opts <- str_c("-", names(opts), "", unlist(opts))
    opts <- str_c(opts, collapse = " ")
  }

  ## move to dir and run 4it2 ppi
  ####################################

  ## make dir to put 4ti2 files in (within the tempdir) timestamped
  dir.create(dir2 <- file.path(dir, timeStamp()))


  ## switch to temporary directory
  oldWd <- getwd()
  setwd(dir2); on.exit(setwd(oldWd), add = TRUE)


  ## run 4ti2
  if (is_mac() || is_unix()) {
  
    system2(
      file.path(get_4ti2_path(), "ppi"),
      paste(opts, N),
      stdout = "ppi_out", 
      stderr = "ppi_err"
    )
    
    # generate shell code
    shell_code <- glue(
      "{file.path(get_4ti2_path(), 'ppi')} {paste(opts, file.path(dir2, 'system'))} > ppi_out 2> ppi_err"
    )
    if(shell) message(shell_code)

  } else if (is_win()) {
    
    system2(
      "cmd.exe",
      glue("/c env.exe {file.path(get_4ti2_path(), 'ppi')} {opts} {N}"),
      stdout = "ppi_out", 
      stderr = "ppi_err"
    )
    
    # generate shell code
    shell_code <- glue(
      "cmd.exe /c env.exe {file.path(get_4ti2_path(), 'ppi')} {opts} {N} > ppi_out 2> ppi_err"
    )
    if(shell) message(shell_code)

  }


  ## print output, if desired
  if(!quiet) cat(readLines("ppi_out"), sep = "\n")


  ## read and return
  t(read.latte(str_c("ppi", N, ".gra")))

}

#' Solve an integer progam with LattE
#'
#' \code{latte_max} and \code{latte_min} use LattE's \code{latte-maximize} and
#' \code{latte-minimize} functions to find the maximum or minimum of a linear
#' objective function over the integers points in a polytope (i.e. satisfying
#' linearity constraints). This makes use of the digging algorithm; see the
#' LattE manual at \url{http://www.math.ucdavis.edu/~latte} for details.
#'
#' @param objective A linear polynomial to pass to [mp()], see examples
#' @param constraints A collection of linear polynomial (in)equalities that
#'   define the feasibility region, the integers in the polytope
#' @param method Method \code{"LP"} or \code{"cones"}
#' @param dir Directory to place the files in, without an ending /
#' @param opts Options; see the LattE manual at
#'   \url{http://www.math.ucdavis.edu/~latte}
#' @param quiet Show latte output
#' @param shell Messages the shell code used to do the computation
#' @param type \code{"max"} or \code{"min"}
#' @return A named list with components \code{par}, a named-vector of optimizing
#'   arguments, and \code{value}, the value of the objective function at the
#'   optimial point.
#' @name latte-optim
#' @examples
#'
#'
#' if (has_latte()) {
#'
#' latte_max(
#'   "-2 x + 3 y", 
#'   c("x + y <= 10", "x >= 0", "y >= 0")
#' )
#' 
#' latte_max(
#'   "-2 x + 3 y", 
#'   c("x + y <= 10", "x >= 0", "y >= 0"),
#'   quiet = FALSE
#' )
#'
#'
#' df <- expand.grid("x" = 0:10, "y" = 0:10)
#' df <- subset(df, x + y <= 10L)
#' df$objective <- with(df, -2*x + 3*y)
#' library("ggplot2")
#' ggplot(df, aes(x, y, size = objective)) +
#'   geom_point()
#'
#' latte_min(
#'   "-2 x + 3 y",
#'   c("x + y <= 10", "x >= 0", "y >= 0"),
#'   method = "cones"
#' )
#'
#'
#'
#' latte_min("-2 x - 3 y - 4 z", c(
#'   "3 x + 2 y + z <= 10",
#'   "2 x + 5 y + 3 z <= 15",
#'   "x >= 0", "y >= 0", "z >= 0"
#' ), "cones", quiet = FALSE)
#'
#'
#'
#'
#'
#' }
#' 
latte_optim <- function(
  objective, 
  constraints, 
  type = c("max", "min"),
  method = c("lp","cones"), 
  dir = tempdir(),
  opts = "",
  quiet = TRUE,
  shell = FALSE
){


  if (!has_latte()) missing_latte_stop()

  type   <- match.arg(type)
  method <- match.arg(method)


  ## set executable to use
  exec <- if(type == "max") "latte-maximize" else "latte-minimize"


  ## parse objective
  if(is.character(objective)) objective <- mp(objective)
  stopifnot(is.linear(objective))


  ## parse constraints into the poly <= 0 format
  nConstraints <- length(constraints)

  if(is.character(constraints) && nConstraints > 1){

  	parsedCons <- as.list(rep(NA, nConstraints))

  	geqNdcs <- which(str_detect(constraints, " >= "))
  	leqNdcs <- which(str_detect(constraints, " <= "))
  	eeqNdcs <- which(str_detect(constraints, " == "))
  	eqNdcs  <- which(str_detect(constraints, " = "))

    if(length(geqNdcs) > 0){
      tmp <- strsplit(constraints[geqNdcs], " >= ")
      parsedCons[geqNdcs] <- lapply(tmp, function(v) mp(v[2]) - mp(v[1]))
    }

    if(length(leqNdcs) > 0){
      tmp <- strsplit(constraints[leqNdcs], " <= ")
      parsedCons[leqNdcs] <- lapply(tmp, function(v) mp(v[1]) - mp(v[2]))
    }

    if(length(eeqNdcs) > 0){
      tmp <- strsplit(constraints[eeqNdcs], " == ")
      parsedCons[eeqNdcs] <- lapply(tmp, function(v) mp(v[1]) - mp(v[2]))
    }

    if(length(eqNdcs) > 0){
      tmp <- strsplit(constraints[eqNdcs], " = ")
      parsedCons[eqNdcs] <- lapply(tmp, function(v) mp(v[1]) - mp(v[2]))
    }

    linearityNdcs <- sort(c(eeqNdcs, eqNdcs))

    constraints <- parsedCons
    class(constraints) <- "mpolyList"

    if (!all(is.linear(constraints))) stop("all polynomials must be linear.", call. = FALSE)
    

  }


  ## mpoly_list_to_mat is in file count.r
  matFull <- mpoly_list_to_mat(c(list(objective), constraints))


  ## make dir to put latte files in (within the tempdir) timestamped
  dir.create(dir2 <- file.path(dir, timeStamp()))


  ## switch to temporary directory
  oldWd <- getwd(); on.exit(setwd(oldWd), add = TRUE)
  setwd(dir2)


  ## convert constraints to latte hrep code and write file
  mat <- cbind(
    -matFull[-1,"coef",drop=FALSE],
    -matFull[-1,-ncol(matFull)]
  )

  if(length(linearityNdcs) > 0) attr(mat, "linearity")   <- linearityNdcs
  
  
  ## note: the nonnegative stuff is built into this
  write.latte(mat, "optim_code")


  ## convert objective to latte hrep code and write file
  mat <- cbind(
    matFull[1,"coef",drop=FALSE],
    matFull[1,-ncol(matFull),drop=FALSE]
  )[,-1, drop = FALSE]
  write.latte(mat, "optim_code.cost")



  ## run latte function
  if (is_mac() || is_unix()) { 
    
    system2(
      file.path(get_latte_path(), exec),
      paste(opts, file.path(dir2, "optim_code")),
      stdout = glue("optim_out"), 
      stderr = glue("optim_err")
    )
    
    # generate shell code
    shell_code <- glue(
      "{file.path(get_latte_path(), exec)} {paste(opts, file.path(dir2, 'optim_code'))} > {exec}_out 2> {exec}_err"
    )
    if(shell) message(shell_code)
    
  } else if(is_win()){ # windows
    
    matFile <- file.path(dir2, "optim_code 2> out.txt")
    matFile <- chartr("\\", "/", matFile)
    matFile <- str_c("/cygdrive/c", str_sub(matFile, 3))
    system(
      paste(
        paste0("cmd.exe /c env.exe"),
        file.path(get_latte_path(), exec),
        opts,
        matFile
      ),
      intern = FALSE, ignore.stderr = FALSE
    )
     
  }
  

  
  ## print count output when quiet = FALSE
  std_out <- readLines("optim_out")
  if(!quiet) cat(std_out, sep = "\n")
  # note: strangely, latte posts non-error info to stderr
  std_err <- readLines("optim_err")
  if(!quiet && any(std_err != "")) message(str_c(std_err, collapse = "\n"))
  
  ## parse output
  if (method == "cones") {
    optimal_solution_string <- std_err[str_which(std_err, "An optimal solution")]
  } else {
    optimal_solution_string <- std_err[str_which(std_err, "A vertex which we found")]
  }

  optimal_solution_string <- str_extract(optimal_solution_string, "\\[-?\\d+( -?\\d+)+\\].?$")
  par <- as.integer(str_extract_all(optimal_solution_string, "-?\\d+")[[1]])
  names(par) <- colnames(matFull)[1:(ncol(matFull)-1)]
  
  val <- str_extract(std_err, "(?<=The optimal value is: )-?\\d+")
  val <- as.integer(val[!is.na(val)])

  ## out
  list(par = par, value = val)
}












#' @rdname latte-optim
#' @export
latte_max <- function(objective, constraints, method = c("lp","cones"),
  dir = tempdir(), opts = "", quiet = TRUE
){

  latte_optim(objective, constraints, "max", method, dir, opts, quiet)

}




#' @rdname latte-optim
#' @export
latte_min <- function(objective, constraints, method = c("lp","cones"),
  dir = tempdir(), opts = "", quiet = TRUE
){

  latte_optim(objective, constraints, "min", method, dir, opts, quiet)

}

library(latte)

## examples from documentation

test_that("latte_count", {
    expect_equal(
        latte_count(c("x + y <= 10", "x >= 0", "y >= 0")),
        66)
})

test_that("latte_max", {
    expect_equal(
        latte_max("-2 x + 3 y", c("x + y <= 10", "x >= 0", "y >= 0")),
        list(par = c(x = 0L, y = 10L), value = 30L))
})

test_that("latte_min", {
    expect_equal(
        latte_min("-2 x + 3 y", c("x + y <= 10", "x >= 0", "y >= 0")),
        list(par = c(x = 10L, y = 0L), value = -20L))
})

A <- genmodel(varlvls = c(2, 2), facets = list(1, 2))

test_that("genmodel", {
    expect_equal(
        A,
        structure(
            c(1L, 0L, 1L, 0L, 0L, 1L, 1L, 0L, 1L, 0L, 0L, 1L, 0L, 1L, 0L, 1L),
            dim = c(4L, 4L)))
})

test_that("markov", {
    expect_equal(
        markov(A, p = "arb"),
        structure(c(1L, -1L, -1L, 1L), dim = c(4L, 1L)))
})

test_that("zbasis", {
    expect_equal(
        zbasis(A, p = "arb"),
        structure(c(-1L, 1L, 1L, -1L), dim = c(4L, 1L)))
})

test_that("groebner", {
    expect_equal(
        groebner(A, p = "arb"),
        structure(c(-1L, 1L, 1L, -1L), dim = c(4L, 1L)))
})

test_that("graver", {
    expect_equal(
        graver(A),
        structure(c(1L, -1L, -1L, 1L), dim = c(4L, 1L)))
})

test_that("zsolve", {
    mat <- rbind(
        c( 1, -1),
        c(-3,  1),
        c( 1,  1)
    )
    rel <- c("<", "<", ">")
    rhs <- c(2, 1, 1)
    sign <- c(0, 1)
    expect_equal(
        zsolve(mat, rel, rhs, sign),
        list(zinhom = structure(c(2L, 0L, 1L, 1L, 0L, 1L, 0L, 1L),
                                dim = c(4L, 2L)),
             zhom = structure(c(1L, 1L, 1L, 3L, 1L, 2L), dim = 3:2)))
})

test_that("qsolve", {
    mat <- rbind(
        c( 1,  1),
        c( 1,  1)
    )
    rel <- c(">", "<")
    sign <- c(0, 0)
    expect_equal(
        qsolve(mat, rel, sign, p = "arb"),
        list(qhom = structure(integer(0), dim = c(0L, 2L)),
             qfree = structure(c(1L, -1L), dim = 1:2)))
})

test_that("ppi", {
    expect_equal(
        ppi(3),
        structure(c(-2L, 1L, 0L, 0L, -3L, 2L, -1L, -1L,
                    1L, -3L, 0L, 1L, 1L, -2L, 1L),
                  dim = c(3L, 5L)))
    expect_equal(
        t(1:3) %*% ppi(3),
        structure(c(0, 0, 0, 0, 0), dim = c(1L, 5L)))
})

#' Solve a linear system over the rationals
#'
#' qsolve runs 4ti2's qsolve program to compute the
#' configuration matrix A corresponding to graphical statistical
#' models given by a simplicial complex and levels on the nodes.
#'
#' @param mat The A matrix (see the 4ti2 documentation or examples)
#' @param rel A vector of "<" or ">" relations
#' @param sign The signs of the individual
#' @param dir Directory to place the files in, without an ending /
#' @param quiet If FALSE, messages the 4ti2 output
#' @param shell Messages the shell code used to do the computation
#' @param ... Additional arguments to pass to the function
#' @return The configuration matrix of the model provided
#' @export
#' @examples
#'
#' if (has_4ti2()) {
#'
#' # x + y > 0
#' # x + y < 0
#'
#' mat <- rbind(
#'   c( 1,  1),
#'   c( 1,  1)
#' )
#' rel <- c(">", "<")
#' sign <- c(0, 0)
#'
#' qsolve(mat, rel, sign, p = "arb")
#' qsolve(mat, rel, sign, p = "arb", quiet = FALSE)
#' qsolve(mat, rel, sign, p = "arb", shell = TRUE)
#' 
#' }
#'
#'
qsolve <- function(mat, rel, sign,
  dir = tempdir(), quiet = TRUE, shell = FALSE, ...
){

  if (!has_4ti2()) missing_4ti2_stop()
  
  ## compute other args
  opts <- as.list(match.call(expand.dots = FALSE))[["..."]]
  if("rhs" %in% names(opts)) stop("qsolve only solve homogeneous systems (b = 0).")
  if(is.null(opts)){
    opts <- ""
  } else {
    opts <- paste0("-", names(opts), "", unlist(opts))
    opts <- paste(opts, collapse = " ")
  }


  ## create and move to dir
  ####################################

  ## make dir to put 4ti2 files in (within the tempdir) timestamped
  dir.create(dir2 <- file.path(dir, timeStamp()))


  ## switch to temporary directory
  oldWd <- getwd()
  setwd(dir2); on.exit(setwd(oldWd), add = TRUE)


  ## arg check
  ####################################

  if(!missing(mat) && !all(is.wholenumber(mat)))
    stop("The entries of mat must all be integers.")

  if(!missing(sign) && !all(is.wholenumber(sign)))
    stop("The entries of sign must all be integers.")

  if(!all(rel %in% c("<", ">")))
    stop("rel must be a vector of \"<\"'s or \">\"'s.")


  ## write files
  ####################################

  if(!missing(mat)) write.latte(mat, "system.mat")
  write.latte(t(rel), "system.rel")
  if(!missing(sign)) write.latte(t(sign), "system.sign")


  ## move to dir and run 4it2 qsolve
  ####################################

  ## run 4ti2
  if (is_mac() || is_unix()) {
    
    system2(
      file.path(get_4ti2_path(), "qsolve"),
      paste(opts, file.path(dir2, "system")),
      stdout = "qsolve_out", 
      stderr = "qsolve_err"
    )
    
    # generate shell code
    shell_code <- glue(
      "{file.path(get_4ti2_path(), 'qsolve')} {paste(opts, file.path(dir2, 'system'))} > qsolve_out 2> qsolve_err"
    )
    if(shell) message(shell_code)

  } else if (is_win()) {

    matFile <- file.path(dir2, "system")
    matFile <- chartr("\\", "/", matFile)
    matFile <- str_c("/cygdrive/c", str_sub(matFile, 3))
    
    system2(
      "cmd.exe",
      glue("/c env.exe {file.path(get_4ti2_path(), 'qsolve')} {opts} {matFile}"),
      stdout = "qsolve_out", 
      stderr = "qsolve_err"
    )
    
    # generate shell code
    shell_code <- glue(
      "cmd.exe /c env.exe {file.path(get_4ti2_path(), 'qsolve')} {opts} {matFile} > qsolve_out 2> qsolve_err"
    )
    if(shell) message(shell_code)

  }


  ## print output, if desired
  if(!quiet) message(paste(readLines("qsolve_out"), "\n"))
  std_err <- readLines("qsolve_err")
  if(any(std_err != "")) warning(str_c(std_err, collapse = "\n"), call. = FALSE)


  ## read and return
  list(
    qhom = read.latte("system.qhom"),
    qfree = read.latte("system.qfree")
  )

}

#' R Interface to LattE and 4ti2
#'
#' Back-end connections to LattE (\url{https://www.math.ucdavis.edu/~latte/})
#' and 4ti2 (\url{http://www.4ti2.de/}) executables and front-end tools
#' facilitating their use in the R ecosystem.
#'
#' @docType package
#' @import mpoly
#' @importFrom ggplot2 ggplot scale_x_continuous scale_y_continuous theme
#'   element_blank theme_bw coord_equal scale_fill_gradient scale_fill_gradient2
#'   aes geom_tile
#' @importFrom magrittr %>%
#' @importFrom stats runif
#' @importFrom utils download.file
#' @importFrom stringr str_detect str_sub str_c str_replace str_replace_all
#'   str_split str_trim str_extract str_extract_all str_which
#' @importFrom dplyr filter mutate arrange
#' @importFrom memoise memoise
#' @importFrom usethis edit_r_environ
#' @importFrom glue glue
#' @name latte
#' @aliases latte package-latte
NULL

#' Count integer points in a polytope
#'
#' \code{latte_count} uses LattE's count function to count the (integer) lattice
#' points in a polytope and compute Ehrhart polynomials.
#'
#' The specification should be one of the following: (1) a character string or
#' strings containing an inequality in the mpoly expression format (see
#' examples), (2) a list of vertices, (3) a list of A and b for the equation Ax
#' <= b (see examples), or (4) raw code for LattE's count program. If a
#' character vector is supplied, (1) and (4) are distinguished by the number of
#' strings.
#'
#' Behind the scenes, count works by writing a latte file and running count on
#' it.  If a specification other than a length one character is given to it
#' (which is considered to be the code), count attempts to convert it into LattE
#' code and then run count on it.
#'
#' @param spec Specification, see details and examples
#' @param dir Directory to place the files in, without an ending /
#' @param quiet Show latte output?
#' @param mpoly When opts = "--ehrhart-polynomial", return the mpoly version of
#'   it
#' @param ... Additional arguments to pass to the function, see count --help at
#'   the command line to see examples.  Note that dashes - should be specified
#'   with underscores _
#' @return The count.  If the count is a number has less than 10 digits, an
#'   integer is returned.  If the number has 10 or more digits, an integer in a
#'   character string is returned. You may want to use the gmp package's as.bigz
#'   to parse it.
#' @name latte-count
#' @examples
#' 
#' if (has_latte()) {
#'
#' spec <- c("x + y <= 10", "x >= 1", "y >= 1")
#' latte_count(spec) # 45
#' latte_count(spec, quiet = FALSE) # 45
#' latte_count(spec, dilation = 10) # 3321
#' latte_count(spec, homog = TRUE) # 45
#'
#' # by default, the output from LattE is in
#' list.files(tempdir())
#' list.files(tempdir(), recursive = TRUE)
#'
#' # ehrhart polynomials
#' latte_count(spec, ehrhart_polynomial = TRUE)
#' latte_count(spec, ehrhart_polynomial = TRUE, mpoly = FALSE)
#'
#' # ehrhart series (raw since mpoly can't handle rational functions)
#' latte_count(spec, ehrhart_series = TRUE)
#'
#' # simplified ehrhart series - not yet implemented
#' #latte_count(spec, simplified_ehrhart_polynomial = TRUE)
#'
#' # first terms of the ehrhart series
#' latte_count(spec, ehrhart_taylor = 1)
#' latte_count(spec, ehrhart_taylor = 2)
#' latte_count(spec, ehrhart_taylor = 3)
#' latte_count(spec, ehrhart_taylor = 4)
#'
#' # multivariate generating function
#' latte_count(spec, multivariate_generating_function = TRUE)
#'
#'
#' # by vertices
#' spec <- list(c(1,1), c(10,1), c(1,10), c(10,10))
#' latte_count(spec)
#' latte_count(spec, vrep = TRUE)
#'
#' code <- "
#' 5 3
#' 1 -1  0
#' 1  0 -1
#' 1 -1 -1
#' 0  1  0
#' 0  0  1
#' "
#' latte_count(code)
#'
#'
#' # for Ax <= b, see this example from the latte manual p.10
#' A <- matrix(c(
#'    1,  0,
#'    0,  1,
#'    1,  1,
#'   -1,  0,
#'    0, -1
#' ), nrow = 5, byrow = TRUE)
#' b <- c(1, 1, 1, 0, 0)
#' latte_count(list(A = A, b = b))
#'
#'
#'
#'
#'
#'
#'
#'
#'
#' }
#' 
#' 






count_core <- function(spec, dir = tempdir(), quiet = TRUE, mpoly = TRUE, ...){

  if (!has_latte()) missing_latte_stop()


  ## initialize specification
  specification <- "unknown"


  ## compute other args
  opts <- as.list(match.call(expand.dots = FALSE))[["..."]]
  if(is.null(opts)){
    opts <- ""
  } else {
    opts_names <- names(opts)
    opts_names <- str_replace_all(opts_names, "_", "-")
    opts <- str_c("--", opts_names, "=", unlist(opts))
    opts <- str_replace_all(opts, "=TRUE", "")
    opts <- str_c(opts, collapse = " ")
  }


  ## look at opts
  if(str_detect(opts, "--ehrhart-series") && mpoly){
    #stop("this option is not yet supported by algstat.", call. = FALSE)
    message("mpoly can't handle rational functions; reverting to raw output.")
    mpoly <- FALSE
  }

  if(str_detect(opts, "--simplified-ehrhart-polynomial")){
    stop("This option is not yet supported by latte.", call. = FALSE)
  }



  ## if the specification is pure code
  if(is.character(spec) && length(spec) == 1){
    specification <- "code"
    code <- spec
  }




  ## if the specification is A and b
  if(is.list(spec) && length(spec) == 2){

    bNegA <- unname(cbind(spec$b, -spec$A))
    spec  <- paste(dim(bNegA), collapse = " ")
    bNegA <- paste(apply(bNegA, 1, paste, collapse = " "), collapse = "\n")
    spec  <- paste(spec, bNegA, sep = "\n")
    spec  <- str_c("\n", spec)

    specification <- "code"
    code <- spec
  }



  ## check for vertex specification
  if(str_detect(opts, "--vrep")) specification <- "vertex"
  if(is.list(spec) && !is.mpolyList(spec) && !str_detect(opts, "--vrep")){

    specification <- "vertex"
    message("Undeclared vertex specification, setting vrep = TRUE.")
    opts <- str_c(opts, "--vrep")

  }




  ## if giving a character string of equations, parse
  ## each to the poly <= 0 format
  # this should transition to eq_to_mp
  if(is.character(spec) && length(spec) > 1){

  	parsedSpec <- as.list(rep(NA, length(spec)))

  	geqNdcs <- which(str_detect(spec, " >= "))
  	leqNdcs <- which(str_detect(spec, " <= "))
  	eeqNdcs <- which(str_detect(spec, " == "))
  	eqNdcs  <- which(str_detect(spec, " = "))

    if(length(geqNdcs) > 0){
      tmp <- strsplit(spec[geqNdcs], " >= ")
      parsedSpec[geqNdcs] <- lapply(tmp, function(v) mp(v[2]) - mp(v[1]))
    }

    if(length(leqNdcs) > 0){
      tmp <- strsplit(spec[leqNdcs], " <= ")
      parsedSpec[leqNdcs] <- lapply(tmp, function(v) mp(v[1]) - mp(v[2]))
    }

    if(length(eeqNdcs) > 0){
      tmp <- strsplit(spec[eeqNdcs], " == ")
      parsedSpec[eeqNdcs] <- lapply(tmp, function(v) mp(v[1]) - mp(v[2]))
    }

    if(length(eqNdcs) > 0){
      tmp <- strsplit(spec[eqNdcs], " = ")
      parsedSpec[eqNdcs] <- lapply(tmp, function(v) mp(v[1]) - mp(v[2]))
    }

    linearityNdcs <- sort(c(eeqNdcs, eqNdcs))

    spec <- parsedSpec
    class(spec) <- "mpolyList"
  }




  ## convert the mpoly specification into a matrix, see latte manual, p. 8
  if(is.mpolyList(spec)){
  	specification <- "hyperplane"
    if(!all(is.linear(spec))){
      stop("All polynomials must be linear.", call. = FALSE)
    }
    mat <- mpoly_list_to_mat(spec)
    mat <- cbind(-mat[,"coef",drop=FALSE], -mat[,-ncol(mat)])

    # convert to code
    code <- paste(nrow(mat), ncol(mat))
    code <- str_c(code, "\n")
    code <- str_c(code,
      paste(apply(unname(mat), 1, paste, collapse = " "),
      collapse = "\n")
    )

    if(length(linearityNdcs) > 0){
      code <- str_c(code, "\n")
      code <- str_c(code,
        str_c("linearity ", length(linearityNdcs), " ",
          paste(linearityNdcs, collapse = " "))
      )
    }
  }




  ## convert vertex specification into a matrix
  if(specification == "vertex"){

  	if(any(!sapply(spec, function(v) length(v) != 1))){
  	  stop(
  	    "Unequal number of coordinates in vertex specification.",
  	    call. = FALSE
  	  )
  	}

    mat <- matrix(unlist(spec), ncol = 2, byrow = TRUE)

    # convert to code
    mat <- cbind(1, mat)
    code <- paste(nrow(mat), ncol(mat))
    code <- str_c(code, "\n")
    code <- str_c(code,
      paste(apply(unname(mat), 1, paste, collapse = " "),
      collapse = "\n")
    )

  }




  ## make dir to put latte files in (within the tempdir) timestamped
  dir.create(dir2 <- file.path(dir, timeStamp()))




  ## write code file
  writeLines(code, con = file.path(dir2, "count_code.latte"))




  ## switch to temporary directory
  oldWd <- getwd()
  setwd(dir2); on.exit(setwd(oldWd), add = TRUE)




  ## run count
  if (is_mac() || is_unix()) {
  
    system2(
      file.path(get_latte_path(), "count"),
      paste(opts, file.path(dir2, "count_code.latte")),
      stdout = "count_out", 
      stderr = "count_err"
    )

  } else if (is_win()) {

    matFile <- file.path(dir2, "count_code.latte")
    matFile <- chartr("\\", "/", matFile)
    matFile <- str_c("/cygdrive/c", str_sub(matFile, 3))

    system2(
      "cmd.exe",
      paste(
        "/c env.exe",
        file.path(get_latte_path(), "count"),
        opts, matFile
      ), 
      stdout = "count_out", 
      stderr = "count_err"
    )

  }




  ## print count output when quiet = FALSE
  if(!quiet) cat(readLines("count_out"), sep = "\n")
  # note: strangely, latte posts non-error info to stderr
  if(!quiet) std_err <- readLines("count_err")
  if(!quiet && any(std_err != "")) message(str_c(std_err, collapse = "\n"))




  ## parse ehrhart polynomial
  if(opts == "--ehrhart-polynomial"){
    outPrint <- readLines("count_out")
    rawPoly <- rev(outPrint)[2]
    if(!mpoly) return(str_trim(rawPoly))
    rawPoly <- str_replace_all(rawPoly, " \\* ", " ")
    if(str_sub(rawPoly, 1, 5) == " + 1 ") rawPoly <- str_sub(rawPoly, 6)
    return(mp(str_trim(rawPoly)))
  }




  ## parse ehrhart series
  if(opts == "--ehrhart-series"){
    outPrint <- readLines("count_code.latte.rat")

    # take off initial "x := " and terminating ":"
    outPrint <- str_sub(outPrint, start = 6, end = nchar(outPrint)-1)

    # return
    return(outPrint)
  }




  ## parse multivariate generating function
  if(opts == "--multivariate-generating-function"){
    outPrint <- readLines("count_code.latte.rat")

    # collapse
    outPrint <- paste(outPrint, collapse = "")

    # return
    if(!mpoly) return(outPrint)

    # change x[0] to vars[1], and so on
    indets <- vars(spec)
    for(k in 1:length(indets)){
      outPrint <- str_replace_all(outPrint, str_c("x\\[",k-1,"\\]"), indets[k])
    }
    return(outPrint)
  }




  ## parse truncated taylor series
  if(str_detect(opts, "--ehrhart-taylor=")){
    outPrint <- readLines("count_out")

    # collapse
    outPrint <- paste(outPrint, collapse = " + ")
    outPrint <- str_replace_all(outPrint, "t", " t")

    # return
    if(!mpoly) return(outPrint)
    return(mp(outPrint))
  }



  ## read in integer and parse if small enough
  out <- readLines("numOfLatticePoints")
  if(nchar(out) < 10) out <- as.integer(out)




  ## print out stats
  if(!quiet){
    cat(readLines("latte_stats"), sep = "\n")
    cat("\n")
  }




  ## out
  out
}

















#' @export
#' @rdname latte-count
latte_count <- memoise::memoise(count_core)





#' @export
#' @rdname latte-count
latte_fcount <- count_core
















mpoly_list_to_mat <- function(mpolyList){
  # this only works for linear mpolyList objects
  vars <- vars(mpolyList)
  varsC <- c(vars, "coef")
  vecMpolyList <- unclass(mpolyList)
  vecMpolyList <- lapply(vecMpolyList, unclass)
  vecMpolyList <- lapply(vecMpolyList, lapply, function(v){
    if(names(v)[1] == "coef") return(v)
    o <- v["coef"]
    names(o) <- names(v)[1]
    o
  })
  vecMpolyList <- lapply(vecMpolyList, unlist)
  vecMpolyList <- lapply(vecMpolyList, function(x){
    varsNeeded <- setdiff(varsC, names(x))
    if(length(varsNeeded) > 0){
      tmp <- rep(0, length(varsNeeded))
      names(tmp) <- varsNeeded
      x <- c(x, tmp)
      x <- x[varsC]
    }
    x
  })
  vecMpolyList <- lapply(vecMpolyList, function(x){
    df <- as.data.frame(t(x))
    row.names(df) <- runif(1) # another way?
    df
  })
  vecMpolyList  <- unsplit(vecMpolyList,
  1:length(vecMpolyList), drop = TRUE)
  row.names(vecMpolyList) <- 1:nrow(vecMpolyList)

  as.matrix(vecMpolyList)
}

#' Unified interface to calling LattE
#'
#' Unified interface to calling LattE
#'
#' @param which_exe The name of LattE executable, e.g. \code{"count"}
#' @param input_file The filename of file to be given to the executable, without
#'   a path
#' @param project_dir The path associated with \code{input_file}.
#' @param opts A named list of optional command line arguments to the
#'   executable.
#' @param stdout,stderr Filenames to redirect standard output and standard error
#'   to.
#' @param shell \code{logical(1)}; should the shell command be messaged to the
#'   user?
#' @return \code{invisible()}
call_latte <- function(
	which_exe,
	input_file,
	project_dir,
	opts = list(),
	stdout = "stdout",
	stderr = "stderr",
	shell = FALSE
) {
	
	
	# translate args to string
	if (length(opts) == 0L) {
		opts <- ""
	} else {
		opts_names <- names(opts)
		opts_names <- str_replace_all(opts_names, "_", "-")
		opts <- str_c("--", opts_names, "=", unlist(opts))
		opts <- str_replace_all(opts, "=TRUE", "")
		opts <- str_c(opts, collapse = " ")
	}
	
	
	#  make call
	if (is_mac() || is_unix()) {
		
		system2(
			file.path(get_latte_path(), which_exe),
			paste(opts, file.path(project_dir, input_file)),
			stdout = stdout, 
			stderr = stderr
		)
		
	} else if (is_win()) {
		
		matFile <- file.path(project_dir, input_file)
		matFile <- chartr("\\", "/", matFile)
		matFile <- str_c("/cygdrive/c", str_sub(matFile, 3))
		
		system2(
			"cmd.exe",
			paste("/c env.exe", file.path(get_latte_path(), which_exe), opts, matFile), 
			stdout = stdout, 
			stderr = stderr
		)
		
	}
	
	
	invisible()
	
}

#' Ones
#'
#' Make an array of ones
#'
#' @param ... A sequence of dimensions separated by commas
#' @return An integer array of ones
#' @export
#' @examples
#'
#' ones(5)
#' ones(5, 1)
#' ones(1, 5)
#' ones(2, 3)
#' ones(2, 3, 2)
#'
#' str(ones(5))
#'
ones <- function(...){
  dims <- as.integer(as.list(match.call(expand.dots = TRUE))[-1])
  a <- rep(1L, prod(dims))
  dim(a) <- dims
  a
}

#' Solve a linear system over the integers
#'
#' zsolve runs 4ti2's zsolve program to compute the configuration matrix A
#' corresponding to graphical statistical models given by a simplicial complex
#' and levels on the nodes.
#'
#' @param mat The A matrix (see the 4ti2 documentation or examples)
#' @param rel A vector of "<" or ">" relations
#' @param rhs The right hand side b
#' @param sign The signs of the individual
#' @param lat A lattice basis (instead of a matrix)
#' @param lb Lower bounds on columns
#' @param ub Upper bounds on columns
#' @param dir Directory to place the files in, without an ending /
#' @param quiet If FALSE, messages the 4ti2 output
#' @param shell Messages the shell code used to do the computation
#' @param ... Additional arguments to pass to the function
#' @return The configuration matrix of the model provided
#' @export
#' @examples
#'
#' if (has_4ti2()) {
#'
#' mat <- rbind(
#'   c( 1, -1),
#'   c(-3,  1),
#'   c( 1,  1)
#' )
#' rel <- c("<", "<", ">")
#' rhs <- c(2, 1, 1)
#' sign <- c(0, 1)
#'
#' zsolve(mat, rel, rhs, sign)
#' zsolve(mat, rel, rhs, sign, quiet = FALSE)
#' zsolve(mat, rel, rhs, sign, shell = TRUE)
#'
#' zsolve(mat, rel, rhs, sign, p = "gmp", quiet = FALSE)
#'
#' }
#'
#' 
zsolve <- function(mat, rel, rhs, sign, lat, lb, ub,
  dir = tempdir(), quiet = TRUE, shell = FALSE, ...
){
  
  if (!has_4ti2()) missing_4ti2_stop()

  ## compute other args
  opts <- as.list(match.call(expand.dots = FALSE))[["..."]]
  if(is.null(opts)){
    opts <- ""
  } else {
    opts <- paste0("-", names(opts), "", unlist(opts))
    opts <- paste(opts, collapse = " ")
  }


  ## create and move to dir
  ####################################

  ## make dir to put 4ti2 files in (within the tempdir) timestamped
  dir.create(dir2 <- file.path(dir, timeStamp()))


  ## switch to temporary directory
  oldWd <- getwd()
  setwd(dir2); on.exit(setwd(oldWd), add = TRUE)


  ## arg check
  ####################################
  if(missing(mat) && missing(lat))
    stop("Either mat or lat must be specified.", call. = FALSE)

  if(!missing(mat) && !all(is.wholenumber(mat)))
    stop("The entries of mat must all be integers.")

  if(!missing(lat) && !all(is.wholenumber(lat)))
    stop("The entries of lat must all be integers.")

  if(!missing(rhs) && !all(is.wholenumber(rhs)))
    stop("The entries of rhs must all be integers.")

  if(!all(rel %in% c("<", ">")))
    stop("rel must be a vector of \"<\"'s or \">\"'s.")


  ## write files
  ####################################

  if(!missing(mat)) write.latte(mat, "system.mat")
  write.latte(t(rel), "system.rel")
  write.latte(t(rhs), "system.rhs")
  if(!missing(sign)) write.latte(t(sign), "system.sign")
  if(missing(mat) && !missing(lat)) write.latte(mat, "system.lat")
  if(!missing(lb)) write.latte(mat, "system.lb")
  if(!missing(ub)) write.latte(mat, "system.ub")


  ## move to dir and run 4it2 zsolve
  ####################################

  ## run 4ti2
  if (is_mac() || is_unix()) {
    
    system2(
      file.path(get_4ti2_path(), "zsolve"),
      paste(opts, file.path(dir2, "system")),
      stdout = "zsolve_out", 
      stderr = "zsolve_err"
    )
    
    # generate shell code
    shell_code <- glue(
      "{file.path(get_4ti2_path(), 'zsolve')} {paste(opts, file.path(dir2, 'system'))} > zsolve_out 2> zsolve_err"
    )
    if(shell) message(shell_code)

  } else if (is_win()) {

    matFile <- file.path(dir2, "system")
    matFile <- chartr("\\", "/", matFile)
    matFile <- str_c("/cygdrive/c", str_sub(matFile, 3))

    system2(
      "cmd.exe",
      glue("/c env.exe {file.path(get_4ti2_path(), 'zsolve')} {opts} {matFile}"),
      stdout = "zsolve_out", 
      stderr = "zsolve_err"
    )
    
    # generate shell code
    shell_code <- glue(
      "cmd.exe /c env.exe {file.path(get_4ti2_path(), 'zsolve')} {opts} {matFile} > zsolve_out 2> zsolve_err"
    )
    if(shell) message(shell_code)

  }


  ## print output, if desired
  if(!quiet) message(paste(readLines("zsolve_out"), "\n"))
  std_err <- readLines("zsolve_err")
  if(any(std_err != "")) warning(str_c(std_err, collapse = "\n"), call. = FALSE)


  ## read and return
  list(
    zinhom = read.latte("system.zinhom"),
    zhom = read.latte("system.zhom")
  )

}

rhash <- function(n) {
  paste(
    sample(c(letters, LETTERS, 0:9), n, replace = TRUE),
    collapse = ""
  )
}
# rhash(10)

timeStamp <- function(){
  timeStamp <- as.character(Sys.time())
  timeStamp <- chartr("-", "_", timeStamp)
  timeStamp <- chartr(" ", "_", timeStamp)
  timeStamp <- chartr(":", "_", timeStamp)
  timeStamp <- paste0(timeStamp, "_", rhash(10))
  timeStamp
}




is.wholenumber <- function(x, tol = .Machine$double.eps^0.5) abs(x - round(x)) < tol




file.path2 <- function(...){
  dots <- list(...)
  if(.Platform$OS.type == "unix"){
    sep <- "/"
  } else {
    sep <- "\\"
  }
  paste0(dots, collapse = sep)
}



is.formula <- function(x) inherits(x, "formula")





is_mac <- function() grepl("darwin", R.version$platform)
is_win <- function() .Platform$OS.type == "windows"
is_linux <- function() (.Platform$OS.type == "unix") && (is_mac() == FALSE)
is_unix <- function() .Platform$OS.type == "unix"
is_solaris <- function() grepl("solaris", R.version$os)








capitalize <- function(s){
  if(length(s) > 1) return(vapply(s, capitalize, character(1)))
  str_c(toupper(str_sub(s, 1, 1)), str_sub(s, 2))
}







`%notin%` <- function(elem, set){
  if(length(elem) > 1) return(vapply(elem, `%notin%`, logical(1), set = set))
  !(elem %in% set)
}

#' Tableau Notation for Markov
#'
#' Print the tableau notation for a Markov move.  See the reference
#' provided, p. 13.
#'
#' @param move a markov move matrix, where the columns are moves in
#'   vector form (e.g. the output of markov)
#' @param dim the dimensions of the table form of the move,
#'   oftentimes a vector of the number of levels of each variable in
#'   order
#' @return an object of class tableau
#' @export tableau
#' @references Drton, M., B. Sturmfels, and S. Sullivant (2009).
#'   \emph{Lectures on Algebraic Statistics}, Basel: Birkhauser
#'   Verlag AG.
#' @examples
#'
#' vec <- matrix(c(1, -1, -1, 1), nrow = 4)
#' varlvls <- c(2, 2)
#' tableau(vec, varlvls)
#'
#'
tableau <- function(move, dim){

  if(is.vector(move) && (is.integer(move) || is.numeric(move)))
	  move <- t(t(move))

  if(ncol(move) == 1){
    p <- length(dim)

    move <- t(t(tab2vec(vec2tab(move, dim))))
    row.names(move) <- str_replace_all(row.names(move), ",", "")

    t1 <- row.names(move)[move > 0]
    t1 <- matrix(
      unlist(lapply(strsplit(t1, ''), as.numeric)),
      ncol = p, byrow = TRUE
    )

    t2 <- row.names(move)[move < 0]
    t2 <- matrix(
      unlist(lapply(strsplit(t2, ''), as.numeric)),
      ncol = p, byrow = TRUE
    )

    out <- list(t1, t2)


  } else {


    out <- list()
    for(k in 1:ncol(move)){
      out[[k]] <- tableau(move[,k,drop = FALSE], dim = dim)
    }

  }



  class(out) <- 'tableau'
  out
}














#' Pretty printing of tableau output.
#'
#' Pretty printing of tableau output.
#'
#' @param x an object of class tableau
#' @param ... ...
#' @usage \method{print}{tableau}(x, ...)
#' @return Invisible string of the printed object.
#' @export
#' @examples
#'
#' # see ?tableau
#'
#'
print.tableau <- function(x, ...){
  if((length(x) == 2) && all(sapply(x, is.matrix))){
    plus <- x[[1]]
    minus <- x[[2]]
    p <- ncol(x[[1]])
    changes <- nrow(x[[1]])
    for(k in 1:changes){
      if(k == floor(changes/2)){
        sep <- ' - '
      } else {
        sep <- '   '
      }
      line2print <- paste(
        paste(plus[k,], collapse = ' '),
        sep,
        paste(minus[k,], collapse = ' '),'\n'
      )
      cat(line2print)
    }
  } else {
    for(k in 1:length(x)){
      print.tableau(x[[k]])
      if(k < length(x)) cat('\n')
    }
  }
}

#' Format/read/write a matrix in latte's style
#'
#' [format_latte()] formats a matrix in latte's style.
#' [write_latte()] writes a latte-formatted file to file.
#' [read_latte()] reads a latte-formatted file from disk.
#'
#' @param mat A matrix
#' @param file A filename
#' @param format "mat" or "Ab"
#' @return \itemize{
#' 
#'   \item [format_latte()] -- A character string of the matrix in
#'   latte format. 
#'   
#'   \item [write_latte()] -- An invisible character
#'   string of the formatted output.
#'   
#'   \item [read_latte()] -- An integer matrix.
#'   
#' }
#' @name latte-files
#' @examples
#'
#'
#' (mat <- matrix(sample(9), 3, 3))
#'
#' format_latte(mat)
#' cat(format_latte(mat))
#'
#' (file <- file.path(tempdir(), "foo.hrep"))
#' write_latte(mat, file)
#' file.show(file)
#' read_latte(file)
#' read_latte(file, "Ab")
#'
#' attr(mat, "linearity") <- c(1, 3)
#' attr(mat, "nonnegative") <- 2
#' mat
#' format_latte(mat)
#' cat(format_latte(mat))
#' write_latte(mat, file)
#' file.show(file)
#' read_latte(file)
#'
#' file.remove(file)
#'
#'
#'






#' @rdname latte-files
#' @export
format_latte <- function(mat, file){

  ## construct file in latte format
  ## e.g. "3 3\n5 6 8\n7 3 4\n2 9 1"
  ## and cat("3 3\n5 6 8\n7 3 4\n2 9 1")
  r <- nrow(mat)
  c <- ncol(mat)
  a <- attributes(mat)
  mat <- apply(mat, 2, format, scientific = FALSE)
  attributes(mat) <- a
  out <- paste(r, c)
  out <- paste0(out, "\n")
  out <- paste0(out,
    paste(
      apply(unname(mat), 1, paste, collapse = " "),
      collapse = "\n"
    )
  )


  ## add linearity lines, if present
  if("linearity" %in% names(attributes(mat))){
    linLines <- attr(mat, "linearity")
    linLineToAdd <- paste(
      "linearity",
      length(linLines),
      paste(linLines, collapse = " ")
    )
    out <- paste(out, linLineToAdd, sep = "\n")
  }


  ## add nonnegative lines, if present
  if("nonnegative" %in% names(attributes(mat))){
    nnegLines <- attr(mat, "nonnegative")
    nnegLineToAdd <- paste(
      "nonnegative",
      length(nnegLines),
      paste(nnegLines, collapse = " ")
    )
    out <- paste(out, nnegLineToAdd, sep = "\n")
  }


  ## return
  out
}





















#' @rdname latte-files
#' @export
write_latte <- function(mat, file){

  ## arg check
  if(missing(file)) stop("file (a filename) must be provided.")


  ## format
  out <- format_latte(mat)


  ## save it to disk and return
  if(!missing(file)) writeLines(out, con = file)


  ## invisibly return code
  invisible(out)
}

#' @rdname latte-files
#' @export
write.latte <- write_latte













#' @rdname latte-files
#' @export
read_latte <- function(file, format = c("mat", "Ab")){

  ## check args
  format <- match.arg(format)


  ## read in file
  ## e.g. [1] "3 3"   "7 2 1" "6 3 4" "9 8 5"
  contents <- readLines(file)


  ## eliminate dimensions
  ## e.g. [1] "7 2 1" "6 3 4" "9 8 5"
  dim <- as.integer(str_split(str_trim(contents[1]), " ")[[1]])


  ## if only one line, return empty int matrix
  if(length(contents) == 1){
    out <- integer(0)
    dim(out) <- dim
    return(out)
  }


  ## split and parse, result is list
  matRows <- lapply(
    str_split(str_trim(contents[2:(1+dim[1])]), " "),
    function(x) as.integer(x[nchar(x) > 0])
  )


  ## put into matrix
  mat <- t(simplify2array(matRows))


  ## check for linearity or nonnegative lines
  if(length(contents) > 1 + dim[1]){

    # isolate the added lines
    addedLines <- contents[-(1:(1 + dim[1]))]

    # look for linearity, assume only one such line
    linQ <- str_detect(addedLines, "linearity")
    if(any(linQ)){
      linLine  <- addedLines[linQ]
      linStuff <- str_replace(linLine, "linearity ", "")
      linNdcs  <- as.integer(str_split(linStuff, " ")[[1]][-1])
      attr(mat, "linearity") <- linNdcs
    }

    # look for nonnegative, assume only one such line
    nnegQ <- str_detect(addedLines, "nonnegative")
    if(any(nnegQ)){
      nnegLine  <- addedLines[nnegQ]
      nnegStuff <- str_replace(nnegLine, "nonnegative ", "")
      nnegNdcs  <- as.integer(str_split(nnegStuff, " ")[[1]][-1])
      attr(mat, "nonnegative") <- nnegNdcs
    }

  }


  ## format
  if(format == "mat"){
    return(mat)
  } else if(format == "Ab"){
    return(list(A = -mat[,-1,drop=FALSE], b = mat[,1]))
  }
}

#' @rdname latte-files
#' @export
read.latte <- read_latte

#' Vector to array conversion
#'
#' Convert a vector into an array given a set of dimensions; it therefore simply
#' wraps [base::aperm()] and [base::array()].
#'
#' This function converts an array (or a multi-way contingency table) into a
#' vector, using a consistent ordering of the cells. The ordering of the cells
#' is lexicographic and cannot be specified by the user.
#'
#' @param vec A vector
#' @param dim The desired array dimensions, oftentimes a vector of the number of
#'   levels of each variable in order
#' @return An array
#' @export
#' @seealso [tab2vec()], [base::aperm()], [base::array()]
#' @examples
#'
#' data(Titanic)
#' str( Titanic )
#' str( tab2vec(Titanic) )
#' 
#' # convert it back into a table (names are removed)
#' vec2tab(
#'   tab2vec(Titanic), 
#'   dim(Titanic)
#' )
#' 
#' # check that they are the same
#' all( vec2tab(tab2vec(Titanic), dim(Titanic)) == Titanic )
#'
#' 
vec2tab <- function(vec, dim){
  aperm(
    array(vec, rev(dim)),
    length(dim):1
  )
}

.onAttach <- function(...) {

  packageStartupMessage('  Please cite latte! See citation("latte") for details.')
  
  if (!has_4ti2()) {
    packageStartupMessage("  - 4ti2 was not set in .Renviron. Use set_4ti2_path() to set it.")
  }
  
  if (!has_latte()) {
    packageStartupMessage("  - LattE was not in .Renviron. Use set_latte_path() to set it.")
  }
  
  invisible(TRUE)
  
}

#' Array to vector conversion
#'
#' Convert an array into a vector.
#'
#' This function converts an array (or a multi-way contingency table) into a
#' vector, using a consistent ordering of the cells. The ordering of the cells
#' is lexicographical and cannot be specified by the user.
#'
#' @param tab An array of counts
#' @return a Named integer vector.  The names correspond to the cell indices in
#'   the table.
#' @export
#' @seealso [vec2tab()]
#' @examples
#'
#' a <- array(1:24, c(2,3,4))
#' tab2vec(a)
#'
#' data(Titanic)
#' tab2vec(Titanic)
#' Titanic[1,1,1,1]
#' Titanic[1,1,1,2]
#'
#' 
tab2vec <- function(tab){

  # if is a vector, return
  if(is.null(dim(tab))){
    tab <- as.vector(tab)
    names(tab) <- 1:length(tab)
    return(tab)
  }

  # if it's a vector already, short-circuit
  if(length(dim(tab)) == 1){
    tab <- as.vector(tab)
    names(tab) <- 1:length(tab)
    return(tab)
  }

  # otherwise, rotate and class
  u <- aperm(tab, length(dim(tab)):1)
  if(class(tab[1]) == "numeric") u <- as.vector(u)
  if(class(tab[1]) == "integer") u <- as.integer(u)

  # create cell indices
  tmpdf <- expand.grid(
    rev.default(lapply(dim(tab), function(x) 1:x))
  )[,length(dim(tab)):1]

  # assign them as names to u
  names(u) <- apply(tmpdf, 1, paste, collapse = ',')

  # return
  u
}

#' Compute a basis with 4ti2
#'
#' 4ti2 provides several executables that can be used to generate bases for a
#' configuration matrix A.  See the references for details.
#'
#' @param A The configuration matrix
#' @param format How the basis (moves) should be returned.  if "mat", the moves
#'   are returned as the columns of a matrix.
#' @param dim The dimension to be passed to [vec2tab()] if format = "tab" is
#'   used; a vector of the number of levels of each variable in order
#' @param all If TRUE, all moves (+ and -) are given.  if FALSE, only the +
#'   moves are given as returned by the executable.
#' @param dir Directory to place the files in, without an ending /
#' @param quiet If FALSE, messages the 4ti2 output
#' @param shell Messages the shell code used to do the computation
#' @param dbName The name of the model in the markov bases database,
#'   http://markov-bases.de, see examples
#' @param ... Additional arguments to pass to the function, e.g. \code{p =
#'   "arb"} specifies the flag \code{-parb}; not setting this issues a common
#'   warning
#' @return a matrix containing the Markov basis as its columns (for easy
#'   addition to tables)
#' @name lattice-bases
#' @references Drton, M., B. Sturmfels, and S. Sullivant (2009). \emph{Lectures
#'   on Algebraic Statistics}, Basel: Birkhauser Verlag AG.
#' @examples
#'
#'
#' if (has_4ti2()) {
#'
#'
#' # basic input and output for the 3x3 independence example
#' (A <- rbind(
#'   kprod(diag(3), ones(1,3)),
#'   kprod(ones(1,3), diag(3))
#' ))
#' markov(A, p = "arb")
#'
#'
#'
#' # you can get the output formatted in different ways:
#' markov(A, p = "arb", all = TRUE)
#' markov(A, p = "arb", "vec")
#' markov(A, p = "arb", "tab", c(3, 3))
#' tableau(markov(A, p = "arb"), dim = c(3, 3)) # tableau notation
#'
#'
#'
#' # you can add options by listing them off
#' # to see the options available to you by function,
#' # go to http://www.4ti2.de
#' markov(A, p = "arb")
#'
#'
#'
#' # the basis functions are automatically cached for future use.
#' # (note that it doesn't persist across sessions.)
#' A <- rbind(
#'   kprod(  diag(4), ones(1,4), ones(1,4)),
#'   kprod(ones(1,4),   diag(4), ones(1,4)),
#'   kprod(ones(1,4), ones(1,4),   diag(4))
#' )
#' system.time(markov(A, p = "arb"))
#' system.time(markov(A, p = "arb"))
#'
#' # the un-cashed versions begin with an "f"
#' # (think: "forgetful" markov)
#' system.time(fmarkov(A, p = "arb"))
#' system.time(fmarkov(A, p = "arb"))
#'
#'
#'
#' # you can see the command line code by typing shell = TRUE
#' # and the standard output wiht quiet = FALSE
#' # we illustrate these with fmarkov because otherwise it's cached
#' (A <- rbind(
#'   kprod(diag(2), ones(1,4)),
#'   kprod(ones(1,4), diag(2))
#' ))
#' fmarkov(A, p = "arb", shell = TRUE)
#' fmarkov(A, p = "arb", quiet = FALSE)
#'
#'
#'
#' # compare the bases for the 3x3x3 no-three-way interaction model
#' A <- rbind(
#'   kprod(  diag(3),   diag(3), ones(1,3)),
#'   kprod(  diag(3), ones(1,3),   diag(3)),
#'   kprod(ones(1,3),   diag(3),   diag(3))
#' )
#' str(  zbasis(A, p = "arb")) #    8 elements = ncol(A) - qr(A)$rank
#' str(  markov(A, p = "arb")) #   81 elements
#' str(groebner(A, p = "arb")) #  110 elements
#' str(  graver(A))            #  795 elements
#'
#'
#' # the other bases are also cached
#' A <- rbind(
#'   kprod(  diag(3), ones(1,3), ones(1,2)),
#'   kprod(ones(1,3),   diag(3), ones(1,2)),
#'   kprod(ones(1,3), ones(1,3),   diag(2))
#' )
#' system.time( graver(A))
#' system.time( graver(A))
#' system.time(fgraver(A))
#' system.time(fgraver(A))
#'
#'
#'
#' # LAS ex 1.2.1, p.12 : 2x3 independence
#' (A <- rbind(
#'   kprod(diag(2), ones(1,3)),
#'   kprod(ones(1,2), diag(3))
#' ))
#'
#' markov(A, p = "arb", "tab", c(3, 3))
#' # Prop 1.2.2 says that there should be
#' 2*choose(2, 2)*choose(3,2) # = 6
#' # moves (up to +-1)
#' markov(A, p = "arb", "tab", c(3, 3), TRUE)
#'
#'
#'
#' # LAS example 1.2.12, p.17  (no 3-way interaction)
#' (A <- rbind(
#'   kprod(  diag(2),   diag(2), ones(1,2)),
#'   kprod(  diag(2), ones(1,2),   diag(2)),
#'   kprod(ones(1,2),   diag(2),   diag(2))
#' ))
#' plot_matrix(A)
#' markov(A, p = "arb")
#' groebner(A, p = "arb")
#' graver(A)
#' tableau(markov(A, p = "arb"), dim = c(2,2,2))
#'
#'
#'
#'
#'
#' # using the markov bases database, must be connected to internet
#' # commented out for predictable and fast cran checks time
#' # A <- markov(dbName = "ind3-3")
#' # B <- markov(rbind(
#' #   kprod(diag(3), ones(1,3)),
#' #   kprod(ones(1,3), diag(3))
#' # ), p = "arb")
#' # all(A == B)
#'
#'
#'
#'
#'
#' # possible issues
#' # markov(diag(1, 10))
#' # zbasis(diag(1, 10), "vec")
#' # groebner(diag(1, 10), "vec", all = TRUE)
#' # graver(diag(1, 10), "vec", all = TRUE)
#' # graver(diag(1, 4), "tab", all = TRUE, dim = c(2,2))
#'
#'
#'
#' }
#'
#' 







#' @param exec don't use this parameter
#' @param memoise don't use this parameter
basis <- function(exec, memoise = TRUE){

  ## stuff in basis
  extension <- switch(exec,
    markov = ".mar",
    groebner = ".gro",
    hilbert = ".hil",
    graver = ".gra",
    zbasis = ".lat",
    zsolve = ".zfree"
  )

  commonName <- switch(exec,
    markov = "markov",
    groebner = "grobner",
    hilbert = "hilbert",
    graver = "graver",
    zbasis = "lattice",
    zsolve = "solve"
  )


  ## memoise or not
  mem_or_not <- if(memoise) memoise::memoise else (function(x) x)


  ## create the function to return
  mem_or_not(function(A, format = c("mat", "vec", "tab"), dim = NULL,
    all = FALSE, dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL,
    ...
  ){

    if (!has_4ti2()) missing_4ti2_stop()


    ## check args
    format <- match.arg(format)
    if (format == "tab" && missing(dim)) {
      stop('If format = "tab" is specified, dim must be also.', call. = FALSE)
    }


    ## compute other args
    opts <- as.list(match.call(expand.dots = FALSE))[["..."]]
    if (is.null(opts)) {
      opts <- ""
    } else {
      opts <- str_c("-", names(opts), "", unlist(opts))
      opts <- str_c(opts, collapse = " ")
    }


    ## make dir to put 4ti2 files in (within the tempdir) timestamped
    dir.create(dir2 <- file.path(dir, timeStamp()))


    ## make 4ti2 file
    if(!missing(A)) write.latte(A, file.path(dir2, "PROJECT.mat"))


    ## switch to temporary directory
    oldWd <- getwd()
    setwd(dir2); on.exit(setwd(oldWd), add = TRUE)


    ## create/retrieve markov basis
    if (is.null(dbName)) {

      ## run 4ti2 if needed
      if (is_mac() || is_unix()) {
        
        system2(
          file.path(get_4ti2_path(), exec),
          paste(opts, file.path(dir2, "PROJECT")),
          stdout = glue("{exec}_out"), 
          stderr = glue("{exec}_err")
        )

        # generate shell code
        shell_code <- glue(
          "{file.path(get_4ti2_path(), exec)} {paste(opts, file.path(dir2, 'PROJECT'))} > {exec}_out 2> {exec}_err"
        )
        if(shell) message(shell_code)

      } else if (is_win()) {

        matFile <- file.path(dir2, "PROJECT")
        matFile <- chartr("\\", "/", matFile)
        matFile <- str_c("/cygdrive/c", str_sub(matFile, 3))

        system2(
          "cmd.exe",
          glue("/c env.exe {file.path(get_4ti2_path(), exec)} {opts} {matFile}"),
          stdout = glue("{exec}_out"), 
          stderr = glue("{exec}_err")
        )

        # generate shell code
        shell_code <- glue(
          "cmd.exe /c env.exe {file.path(get_4ti2_path(), exec)} {opts} {matFile} > {exec}_out 2> {exec}_err"
        )
        if(shell) message(shell_code)

      }


      if(!quiet) cat(readLines(glue("{exec}_out")), sep = "\n")
      std_err <- readLines(glue("{exec}_err"))
      if(any(std_err != "")) warning(str_c(std_err, collapse = "\n"), call. = FALSE)

    } else { # if the model name is specified

      download.file(
        paste0("http://markov-bases.de/data/", dbName, "/", dbName, extension),
        destfile = "PROJECT.mar" # already in tempdir
      )

    }

    ## fix case of no graver basis
    if(exec == "graver"){
      if(paste0("PROJECT", extension) %notin% list.files(dir2)){
        warning(sprintf("%s basis empty, returning 0's.", capitalize(commonName)), call. = FALSE)
        return(fix_graver(A, format, dim))
      }
    }


    ## figure out what files to keep them, and make 4ti2 object
    basis <- t(read.latte(paste0("PROJECT", extension)))


    ## fix case of no basis
    basisDim <- dim(basis)
    noBasisFlag <- FALSE
    if(any(basisDim == 0)){
      noBasisFlag <- TRUE
      warning(sprintf("%s basis empty, returning 0's.", capitalize(commonName)), call. = FALSE)
      basisDim[basisDim == 0] <- 1L
      basis <- rep(0L, prod(basisDim))
      dim(basis) <- basisDim
    }


    ## format
    if(all && !noBasisFlag) basis <- cbind(basis, -basis)


    # out
    if(format == "mat"){
      return(basis)
    } else {
      lbasis <- as.list(rep(NA, ncol(basis)))
      for(k in 1:ncol(basis)) lbasis[[k]] <- basis[,k]
      if(format == "vec") return(lbasis)
      if(format == "tab") return(lapply(lbasis, vec2tab, dim = dim))
    }
  })
}





# #' @export
# #' @rdname lattice-bases
# zsolve <- basis("zsolve", memoise = TRUE)

#' @export
#' @rdname lattice-bases
zbasis <- basis("zbasis", memoise = TRUE)

#' @export
#' @rdname lattice-bases
markov <- basis("markov", memoise = TRUE)

#' @export
#' @rdname lattice-bases
groebner <- basis("groebner", memoise = TRUE)

#' @export
#' @rdname lattice-bases
hilbert <- basis("hilbert", memoise = TRUE)

#' @export
#' @rdname lattice-bases
graver <- basis("graver", memoise = TRUE)






# #' @export
# #' @rdname lattice-bases
# fzsolve <- basis("zsolve", memoise = FALSE)

#' @export
#' @rdname lattice-bases
fzbasis <- basis("zbasis", memoise = FALSE)

#' @export
#' @rdname lattice-bases
fmarkov <- basis("markov", memoise = FALSE)

#' @export
#' @rdname lattice-bases
fgroebner <- basis("groebner", memoise = FALSE)

#' @export
#' @rdname lattice-bases
fhilbert <- basis("hilbert", memoise = FALSE)

#' @export
#' @rdname lattice-bases
fgraver <- basis("graver", memoise = FALSE)




fix_graver <- function(A, format, dim){
  if(format == "mat"){
    return(matrix(0L, nrow = ncol(A)))
  } else {
    lbasis <- list(rep(0L, ncol(A)))
    if(format == "vec") return(lbasis)
    if(format == "tab") return(lapply(lbasis, vec2tab, dim = dim))
  }
}

#' Plot a matrix
#'
#' plot_matrix is a R variant of Matlab's \code{spy} function.
#'
#' @param A A matrix
#' @return a ggplot object
#' @author David Kahle \email{david@@kahle.io}
#' @name plot-matrix
#' @examples
#'
#' # the no-three-way interaction configuration
#' (A <- kprod(ones(1,3), diag(3), ones(3)))
#' plot_matrix(A)
#' 
#' 
#' if (has_4ti2()) {
#' 
#' plot_matrix(markov(A))
#' 
#' (A <- genmodel(c(2L, 2L), list(1L, 2L)))
#' plot_matrix(A)
#' plot_matrix(markov(A))
#' 
#' (A <- genmodel(c(5L, 5L), list(1L, 2L)))
#' plot_matrix(A)
#' plot_matrix(markov(A))
#' 
#' }
#'






#' @rdname plot-matrix
#' @export
plot_matrix <- function(A){
  x <- NULL; rm(x)
  y <- NULL; rm(y)

  low <- if(any(A < 0)){
    low <- "blue"; high <- "red"
    fillScale <- scale_fill_gradient2(
      low = "blue", mid = "grey80",
      high = "red", midpoint = 0, guide = FALSE, space = "Lab"
    )
  } else {
    fillScale <- scale_fill_gradient(
      low = "white", high = "black", guide = FALSE
    )
  }
  df <- expand.grid(x = 1:ncol(A), y = 1:nrow(A))
  B <- t(A)
  df$A <- as.integer(B[,ncol(B):1])
  ggplot(df, aes(x, y, fill = A)) +
    geom_tile() +
    fillScale +
    theme_bw() + coord_equal() +
    scale_x_continuous(expand = c(0,0)) +
    scale_y_continuous(expand = c(0,0)) +
    theme(
      line = element_blank(), text = element_blank(),
      plot.margin = grid::unit(c(0, 0, 0, 0), "lines")
    )
}

#' Generate a configuration matrix
#'
#' genmodel runs 4ti2's genmodel program to compute the configuration matrix A
#' corresponding to graphical statistical models given by a simplicial complex
#' and levels on the nodes.
#'
#' @param varlvls a vector containing the number of levels of each variable
#' @param facets the facets generating the hierarchical model, a list of vectors
#'   of variable indices
#' @param dir Directory to place the files in, without an ending /
#' @param quiet If FALSE, messages the 4ti2 output
#' @param shell Messages the shell code used to do the computation
#' @param ... Additional arguments to pass to the function
#' @return The configuration matrix of the model provided
#' @export
#' @examples
#'
#' if (has_4ti2()) {
#'
#' varlvls <- rep(2, 2)
#' facets <- list(1, 2)
#' genmodel(varlvls, facets)
#' genmodel(varlvls, facets, quiet = FALSE)
#'
#' varlvls <- rep(3, 3)
#' facets <- list(1:2, 2:3, c(3,1))
#' genmodel(varlvls, facets)
#'
#' # compare this to algstat's hmat function
#'
#' }
#' 
genmodel <- function(varlvls, facets, dir = tempdir(), quiet = TRUE,
    shell = FALSE, ...
){
  
  if (!has_4ti2()) missing_4ti2_stop()
  

  ## compute other args
  opts <- as.list(match.call(expand.dots = FALSE))[["..."]]
  if (is.null(opts)) {
    opts <- ""
  } else {
    opts <- str_c("-", names(opts), "", unlist(opts))
    opts <- str_c(opts, collapse = " ")
  }


  ## compute/write 4ti2 code file
  ####################################

  ## format varlvls
  formatted_varlvls <- str_sub(format_latte(t(varlvls)), 3)

  ## format simplices
  formatted_facets <- vapply(
    facets,
    function(v) paste(length(v), paste(v, collapse = " ")),
    character(1)
  )
  formatted_facets <- paste(
    length(facets),
    paste(formatted_facets, collapse = "\n"),
    sep = "\n"
  )

  ## bring the two together
  code <- paste(formatted_varlvls, formatted_facets, sep = "\n")


  ## make dir to put 4ti2 files in (within the tempdir) timestamped
  dir.create(dir2 <- file.path(dir, timeStamp()))


  ## make 4ti2 file
  writeLines(code, con = file.path(dir2, "PROJECT.mod"))


  ## move to dir and run 4it2 genmodel
  ####################################


  ## switch to temporary directory
  oldWd <- getwd()
  setwd(dir2); on.exit(setwd(oldWd), add = TRUE)


  ## run 4ti2
  if (is_mac() || is_unix()) {

    system2(
      file.path(get_4ti2_path(), "genmodel"),
      paste(opts, file.path(dir2, "PROJECT")),
      stdout = "genmodel_out", 
      stderr = "genmodel_err"
    )

    # generate shell code
    shell_code <- glue(
      "{file.path(get_4ti2_path(), 'genmodel')} {paste(opts, file.path(dir2, 'PROJECT'))} > genmodel_out 2> genmodel_err"
    )
    if(shell) message(shell_code)

  } else if (is_win()) {

    matFile <- file.path(dir2, "PROJECT")
    matFile <- chartr("\\", "/", matFile)
    matFile <- str_c("/cygdrive/c", str_sub(matFile, 3))

    system2(
      "cmd.exe",
      glue("/c env.exe {file.path(get_4ti2_path(), 'genmodel')} {opts} {matFile}"),
      stdout = "genmodel_out", 
      stderr = "genmodel_err"
    )

    # generate shell code
    shell_code <- glue(
      "cmd.exe /c env.exe {file.path(get_4ti2_path(), 'genmodel')} {opts} {matFile} > genmodel_out 2> genmodel_err"
    )
    if(shell) message(shell_code)

  }


  ## print output, if desired
  if(!quiet) message(paste0(readLines("genmodel_out"), "\n"))
  std_err <- readLines("genmodel_err")
  if(any(std_err != "")) warning(str_c(std_err, collapse = "\n"), call. = FALSE)

  ## read and return
  read.latte(paste0("PROJECT", ".mat"))

}

#' Set paths to LattE and 4ti2 executables
#'
#' These are helper functions that deal with pathing to external programs and
#' asking if they are present. When latte is loaded it attempts to find LattE
#' and 4ti2 executables by looking for environment variables specifying them,
#' i.e. their paths as specified in your `.Renviron` file.
#'
#' For easiest use, you'll want to specify the paths of LattE and 4ti2
#' executables in your `~/.Renviron` file. They should look something like
#'
#' `LATTE=/Applications/latte/bin`
#'
#' `FOURTITWO=/Applications/latte/bin`
#'
#' You can set these permanently with [usethis::edit_r_environ()].
#'
#' You can change these for the current session using, e.g., [set_latte_path()],
#' which accepts a character string or, if missing, uses [file.choose()] to let
#' you interactively; you just select an arbitrary executable.
#'
#' @param path A character string, the path to a 4ti2 function (e.g. `markov`)
#'   for setting 4ti2's path or a LattE function (e.g. `count`) for LattE's path
#' @return A logical(1) or character(1) containing the path.
#' @name pathing
#' @author David Kahle \email{david@@kahle.io}
#' @examples
#'
#'
#' has_4ti2()
#' if (has_4ti2()) get_4ti2_path()
#'
#' has_latte()
#' if (has_4ti2()) get_latte_path()
#'
#'
#' # you can set these paths permanently with the following. note that you'll 
#' # need to re-start the R session afterwards or simply pass the path into, 
#' # e.g., set_latte_path(). see below for more details on that.
#' if (interactive()) edit_r_environ()
#'
#'
#' # you can change these in your current session with set_latte_path() and
#' if (had_latte <- has_latte()) old_latte_path <- get_latte_path()
#' set_latte_path("/path/to/latte")
#' get_latte_path()
#'
#' if (had_latte) set_latte_path(old_latte_path)
#' get_latte_path()
#' 
NULL












#' @rdname pathing
#' @export
set_latte_path <- function(path){

  if(missing(path) && interactive()){

    latte_path <- dirname(file.choose())
    if(is_win() && str_detect(latte_path,"C:/")){
      latte_path <- str_replace(dirname(latte_path), "C:/", "/cygdrive/c/")
    }
    Sys.setenv("LATTE" = latte_path)
    return(invisible(latte_path))

  } else if(!missing(path)){

    Sys.setenv("LATTE" = path)
    return(invisible(path))

  } else {
    
    stop(
      "If the session is not interactive, a path must be specified.",
      call. = FALSE
    )
    
  }
}













#' @rdname pathing
#' @export
set_4ti2_path <- function(path){

  if(missing(path) && interactive()){

    `4ti2_path` <- dirname(file.choose())
    if(is_win() && str_detect(`4ti2_path`,"C:/")){
      `4ti2_path` <- str_replace(`4ti2_path`, "C:/", "/cygdrive/c/")
    }
    Sys.setenv("FOURTITWO" = `4ti2_path`)
    return(invisible(`4ti2_path`))

  } else if(!missing(path)){

    Sys.setenv("FOURTITWO" = path)
    return(invisible(path))

  } else {
    
    stop(
      "If the session is not interactive, a path must be specified.",
      call. = FALSE
    )
    
  }
}









#' @rdname pathing
#' @export
get_4ti2_path <- function() {
	if (Sys.getenv("FOURTITWO") != "") {
		Sys.getenv("FOURTITWO")
	} else {
		Sys.getenv("4TI2")
	}
}



#' @rdname pathing
#' @export
get_latte_path <- function() Sys.getenv("LATTE")



#' @rdname pathing
#' @export
has_4ti2 <- function() get_4ti2_path() != ""



#' @rdname pathing
#' @export
has_latte <- function() get_latte_path() != ""



#' @rdname pathing
#' @export
missing_4ti2_stop <- function() {
  stop(
    "latte doesn't know where 4ti2 is.\n", 
    "See ?set_4ti2_path to learn how to set it.", 
    call. = FALSE
  )
}



#' @rdname pathing
#' @export
missing_latte_stop <- function() {
  stop(
    "latte doesn't know where LattE is.\n", 
    "See ?set_latte_path to learn how to set it.", 
    call. = FALSE
  )
}



#' @importFrom usethis edit_r_environ
#' @export
usethis::edit_r_environ

#' Iterated Kronecker product
#'
#' Compute the Kronecker product of several matrices.
#'
#' If kronecker is  the function that computes A x B, kprod computes A x B x C
#' and so on; it's a wrapper of Reduce and kronecker.
#'
#' @param ... A listing of matrices
#' @param FUN A function to pass to [kronecker()]
#' @return A matrix that is the kronecker product of the specified matrices
#'   (from left to right).
#' @export
#' @examples
#'
#' kprod(diag(2), t(ones(2)))
#' kprod(t(ones(2)), diag(2))
#'
#'
#' kprod(diag(2), t(ones(2)), t(ones(2)))
#' kprod(t(ones(2)), diag(2), t(ones(2)))
#' kprod(t(ones(2)), t(ones(2)), diag(2))
#'
#'
#' # cf. aoki, hara, and takemura p.13
#' rbind(
#'   kprod(diag(2), t(ones(2))),
#'   kprod(t(ones(2)), diag(2))
#' ) 
#'
#' 
kprod <- function(..., FUN = `*`) {
  Reduce(
    function(X, Y) kronecker(X, Y, FUN),
    list(...)
  )
}

# Copyright (c) 1998 Zanshin Inc.             <http://www.zanshin.com>
# 
# The contents of this file are subject to the Zanshin Public License Version
# 1.0 (the "License"); you may not use this file except in compliance with the
# License.  You should have received a copy of the License with Latte; see
# the file COPYING.  You may also obtain a copy of the License at
# <http://www.zanshin.com/ZPL.html>.
# 
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
# 
# The Original Code is Latte.
# 
# The Initial Developer of the Original Code is Zanshin, Inc.

LATTE_PATH=$srcdir/..
export LATTE_PATH

name=`basename $0`

rm -f $name.log
../latte-html --fragment $srcdir/$name.latte > $name.log 2>&1
result=$?

case $name in
 *-x) expfail=1 ;;
 *) expfail=0 ;;
esac

if test $expfail -eq 0
then
  if test $result -eq 0
  then
    ok=1
  else
    ok=0
  fi
else
  if test $result -eq 0
  then
    ok=0
  else
    ok=1
  fi
fi

if test $ok -eq 1
then
    rm -f $name.out
    sed -e "s/@VERSION@/$version/g" \
        -e "s,@URL@,$url,g" \
        -e "s,@SRCDIR@,$srcdir,g" $srcdir/$name.exp > $name.out
    if cmp -s $name.log $name.out > /dev/null 2>&1
    then
	echo "*** Test $name SUCCEEDED" >> log
	rm -f $name.log $name.out
	exit 0
    else
	echo "*** Test $name FAILED" >> log
	echo "**** Expected output was" >> log
	cat $name.out >> log
	echo " " >> log
	echo "**** Actual output was" >> log
	cat $name.log >> log
	echo " " >> log
	rm -f $name.log $name.out
	exit 1
    fi
else
    echo "*** Test $name FAILED" >> log
    cat $name.log >> log
    echo " " >> log
    rm -f $name.log
    exit 2
fi

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lattice-bases.R
\name{lattice-bases}
\alias{lattice-bases}
\alias{basis}
\alias{zbasis}
\alias{markov}
\alias{groebner}
\alias{hilbert}
\alias{graver}
\alias{fzbasis}
\alias{fmarkov}
\alias{fgroebner}
\alias{fhilbert}
\alias{fgraver}
\title{Compute a basis with 4ti2}
\usage{
basis(exec, memoise = TRUE)

zbasis(A, format = c("mat", "vec", "tab"), dim = NULL, all = FALSE,
  dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL, ...)

markov(A, format = c("mat", "vec", "tab"), dim = NULL, all = FALSE,
  dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL, ...)

groebner(A, format = c("mat", "vec", "tab"), dim = NULL, all = FALSE,
  dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL, ...)

hilbert(A, format = c("mat", "vec", "tab"), dim = NULL, all = FALSE,
  dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL, ...)

graver(A, format = c("mat", "vec", "tab"), dim = NULL, all = FALSE,
  dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL, ...)

fzbasis(A, format = c("mat", "vec", "tab"), dim = NULL, all = FALSE,
  dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL, ...)

fmarkov(A, format = c("mat", "vec", "tab"), dim = NULL, all = FALSE,
  dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL, ...)

fgroebner(A, format = c("mat", "vec", "tab"), dim = NULL,
  all = FALSE, dir = tempdir(), quiet = TRUE, shell = FALSE,
  dbName = NULL, ...)

fhilbert(A, format = c("mat", "vec", "tab"), dim = NULL, all = FALSE,
  dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL, ...)

fgraver(A, format = c("mat", "vec", "tab"), dim = NULL, all = FALSE,
  dir = tempdir(), quiet = TRUE, shell = FALSE, dbName = NULL, ...)
}
\arguments{
\item{exec}{don't use this parameter}

\item{memoise}{don't use this parameter}

\item{A}{The configuration matrix}

\item{format}{How the basis (moves) should be returned.  if "mat", the moves
are returned as the columns of a matrix.}

\item{dim}{The dimension to be passed to \code{\link[=vec2tab]{vec2tab()}} if format = "tab" is
used; a vector of the number of levels of each variable in order}

\item{all}{If TRUE, all moves (+ and -) are given.  if FALSE, only the +
moves are given as returned by the executable.}

\item{dir}{Directory to place the files in, without an ending /}

\item{quiet}{If FALSE, messages the 4ti2 output}

\item{shell}{Messages the shell code used to do the computation}

\item{dbName}{The name of the model in the markov bases database,
http://markov-bases.de, see examples}

\item{...}{Additional arguments to pass to the function, e.g. \code{p =
  "arb"} specifies the flag \code{-parb}; not setting this issues a common
warning}
}
\value{
a matrix containing the Markov basis as its columns (for easy
addition to tables)
}
\description{
4ti2 provides several executables that can be used to generate bases for a
configuration matrix A.  See the references for details.
}
\examples{


if (has_4ti2()) {


# basic input and output for the 3x3 independence example
(A <- rbind(
  kprod(diag(3), ones(1,3)),
  kprod(ones(1,3), diag(3))
))
markov(A, p = "arb")



# you can get the output formatted in different ways:
markov(A, p = "arb", all = TRUE)
markov(A, p = "arb", "vec")
markov(A, p = "arb", "tab", c(3, 3))
tableau(markov(A, p = "arb"), dim = c(3, 3)) # tableau notation



# you can add options by listing them off
# to see the options available to you by function,
# go to http://www.4ti2.de
markov(A, p = "arb")



# the basis functions are automatically cached for future use.
# (note that it doesn't persist across sessions.)
A <- rbind(
  kprod(  diag(4), ones(1,4), ones(1,4)),
  kprod(ones(1,4),   diag(4), ones(1,4)),
  kprod(ones(1,4), ones(1,4),   diag(4))
)
system.time(markov(A, p = "arb"))
system.time(markov(A, p = "arb"))

# the un-cashed versions begin with an "f"
# (think: "forgetful" markov)
system.time(fmarkov(A, p = "arb"))
system.time(fmarkov(A, p = "arb"))



# you can see the command line code by typing shell = TRUE
# and the standard output wiht quiet = FALSE
# we illustrate these with fmarkov because otherwise it's cached
(A <- rbind(
  kprod(diag(2), ones(1,4)),
  kprod(ones(1,4), diag(2))
))
fmarkov(A, p = "arb", shell = TRUE)
fmarkov(A, p = "arb", quiet = FALSE)



# compare the bases for the 3x3x3 no-three-way interaction model
A <- rbind(
  kprod(  diag(3),   diag(3), ones(1,3)),
  kprod(  diag(3), ones(1,3),   diag(3)),
  kprod(ones(1,3),   diag(3),   diag(3))
)
str(  zbasis(A, p = "arb")) #    8 elements = ncol(A) - qr(A)$rank
str(  markov(A, p = "arb")) #   81 elements
str(groebner(A, p = "arb")) #  110 elements
str(  graver(A))            #  795 elements


# the other bases are also cached
A <- rbind(
  kprod(  diag(3), ones(1,3), ones(1,2)),
  kprod(ones(1,3),   diag(3), ones(1,2)),
  kprod(ones(1,3), ones(1,3),   diag(2))
)
system.time( graver(A))
system.time( graver(A))
system.time(fgraver(A))
system.time(fgraver(A))



# LAS ex 1.2.1, p.12 : 2x3 independence
(A <- rbind(
  kprod(diag(2), ones(1,3)),
  kprod(ones(1,2), diag(3))
))

markov(A, p = "arb", "tab", c(3, 3))
# Prop 1.2.2 says that there should be
2*choose(2, 2)*choose(3,2) # = 6
# moves (up to +-1)
markov(A, p = "arb", "tab", c(3, 3), TRUE)



# LAS example 1.2.12, p.17  (no 3-way interaction)
(A <- rbind(
  kprod(  diag(2),   diag(2), ones(1,2)),
  kprod(  diag(2), ones(1,2),   diag(2)),
  kprod(ones(1,2),   diag(2),   diag(2))
))
plot_matrix(A)
markov(A, p = "arb")
groebner(A, p = "arb")
graver(A)
tableau(markov(A, p = "arb"), dim = c(2,2,2))





# using the markov bases database, must be connected to internet
# commented out for predictable and fast cran checks time
# A <- markov(dbName = "ind3-3")
# B <- markov(rbind(
#   kprod(diag(3), ones(1,3)),
#   kprod(ones(1,3), diag(3))
# ), p = "arb")
# all(A == B)





# possible issues
# markov(diag(1, 10))
# zbasis(diag(1, 10), "vec")
# groebner(diag(1, 10), "vec", all = TRUE)
# graver(diag(1, 10), "vec", all = TRUE)
# graver(diag(1, 4), "tab", all = TRUE, dim = c(2,2))



}


}
\references{
Drton, M., B. Sturmfels, and S. Sullivant (2009). \emph{Lectures
on Algebraic Statistics}, Basel: Birkhauser Verlag AG.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppi.R
\name{ppi}
\alias{ppi}
\title{Compute the primitive partition identities}
\usage{
ppi(N, dir = tempdir(), quiet = TRUE, shell = FALSE, ...)
}
\arguments{
\item{N}{A postive integer > 2}

\item{dir}{Directory to place the files in, without an ending /}

\item{quiet}{If FALSE, messages the 4ti2 output}

\item{shell}{Messages the shell code used to do the computation}

\item{...}{Additional arguments to pass to the function}
}
\value{
A matrix containing the basis as its columns (for easy addition to
tables)
}
\description{
ppi runs 4ti2's ppi program to compute the primitive partition identities,
that is, the Graver basis of 1:N.
}
\examples{

if (has_4ti2()) {

ppi(3)
t(ppi(3)) \%*\% 1:3
plot_matrix(ppi(3))

graver(t(1:3))
plot_matrix(graver(t(1:3)))

ppi(5, quiet = FALSE, shell = TRUE)

}


}
\seealso{
\code{\link[=graver]{graver()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableau.R
\name{tableau}
\alias{tableau}
\title{Tableau Notation for Markov}
\usage{
tableau(move, dim)
}
\arguments{
\item{move}{a markov move matrix, where the columns are moves in
vector form (e.g. the output of markov)}

\item{dim}{the dimensions of the table form of the move,
oftentimes a vector of the number of levels of each variable in
order}
}
\value{
an object of class tableau
}
\description{
Print the tableau notation for a Markov move.  See the reference
provided, p. 13.
}
\examples{

vec <- matrix(c(1, -1, -1, 1), nrow = 4)
varlvls <- c(2, 2)
tableau(vec, varlvls)


}
\references{
Drton, M., B. Sturmfels, and S. Sullivant (2009).
\emph{Lectures on Algebraic Statistics}, Basel: Birkhauser
Verlag AG.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ones.R
\name{ones}
\alias{ones}
\title{Ones}
\usage{
ones(...)
}
\arguments{
\item{...}{A sequence of dimensions separated by commas}
}
\value{
An integer array of ones
}
\description{
Make an array of ones
}
\examples{

ones(5)
ones(5, 1)
ones(1, 5)
ones(2, 3)
ones(2, 3, 2)

str(ones(5))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latte-count.R
\name{latte-count}
\alias{latte-count}
\alias{count_core}
\alias{latte_count}
\alias{latte_fcount}
\title{Count integer points in a polytope}
\usage{
count_core(spec, dir = tempdir(), quiet = TRUE, mpoly = TRUE, ...)

latte_count(spec, dir = tempdir(), quiet = TRUE, mpoly = TRUE, ...)

latte_fcount(spec, dir = tempdir(), quiet = TRUE, mpoly = TRUE, ...)
}
\arguments{
\item{spec}{Specification, see details and examples}

\item{dir}{Directory to place the files in, without an ending /}

\item{quiet}{Show latte output?}

\item{mpoly}{When opts = "--ehrhart-polynomial", return the mpoly version of
it}

\item{...}{Additional arguments to pass to the function, see count --help at
the command line to see examples.  Note that dashes - should be specified
with underscores _}
}
\value{
The count.  If the count is a number has less than 10 digits, an
integer is returned.  If the number has 10 or more digits, an integer in a
character string is returned. You may want to use the gmp package's as.bigz
to parse it.
}
\description{
\code{latte_count} uses LattE's count function to count the (integer) lattice
points in a polytope and compute Ehrhart polynomials.
}
\details{
The specification should be one of the following: (1) a character string or
strings containing an inequality in the mpoly expression format (see
examples), (2) a list of vertices, (3) a list of A and b for the equation Ax
<= b (see examples), or (4) raw code for LattE's count program. If a
character vector is supplied, (1) and (4) are distinguished by the number of
strings.

Behind the scenes, count works by writing a latte file and running count on
it.  If a specification other than a length one character is given to it
(which is considered to be the code), count attempts to convert it into LattE
code and then run count on it.
}
\examples{

if (has_latte()) {

spec <- c("x + y <= 10", "x >= 1", "y >= 1")
latte_count(spec) # 45
latte_count(spec, quiet = FALSE) # 45
latte_count(spec, dilation = 10) # 3321
latte_count(spec, homog = TRUE) # 45

# by default, the output from LattE is in
list.files(tempdir())
list.files(tempdir(), recursive = TRUE)

# ehrhart polynomials
latte_count(spec, ehrhart_polynomial = TRUE)
latte_count(spec, ehrhart_polynomial = TRUE, mpoly = FALSE)

# ehrhart series (raw since mpoly can't handle rational functions)
latte_count(spec, ehrhart_series = TRUE)

# simplified ehrhart series - not yet implemented
#latte_count(spec, simplified_ehrhart_polynomial = TRUE)

# first terms of the ehrhart series
latte_count(spec, ehrhart_taylor = 1)
latte_count(spec, ehrhart_taylor = 2)
latte_count(spec, ehrhart_taylor = 3)
latte_count(spec, ehrhart_taylor = 4)

# multivariate generating function
latte_count(spec, multivariate_generating_function = TRUE)


# by vertices
spec <- list(c(1,1), c(10,1), c(1,10), c(10,10))
latte_count(spec)
latte_count(spec, vrep = TRUE)

code <- "
5 3
1 -1  0
1  0 -1
1 -1 -1
0  1  0
0  0  1
"
latte_count(code)


# for Ax <= b, see this example from the latte manual p.10
A <- matrix(c(
   1,  0,
   0,  1,
   1,  1,
  -1,  0,
   0, -1
), nrow = 5, byrow = TRUE)
b <- c(1, 1, 1, 0, 0)
latte_count(list(A = A, b = b))









}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latte-optim.R
\name{latte-optim}
\alias{latte-optim}
\alias{latte_optim}
\alias{latte_max}
\alias{latte_min}
\title{Solve an integer progam with LattE}
\usage{
latte_optim(objective, constraints, type = c("max", "min"),
  method = c("lp", "cones"), dir = tempdir(), opts = "",
  quiet = TRUE, shell = FALSE)

latte_max(objective, constraints, method = c("lp", "cones"),
  dir = tempdir(), opts = "", quiet = TRUE)

latte_min(objective, constraints, method = c("lp", "cones"),
  dir = tempdir(), opts = "", quiet = TRUE)
}
\arguments{
\item{objective}{A linear polynomial to pass to \code{\link[=mp]{mp()}}, see examples}

\item{constraints}{A collection of linear polynomial (in)equalities that
define the feasibility region, the integers in the polytope}

\item{type}{\code{"max"} or \code{"min"}}

\item{method}{Method \code{"LP"} or \code{"cones"}}

\item{dir}{Directory to place the files in, without an ending /}

\item{opts}{Options; see the LattE manual at
\url{http://www.math.ucdavis.edu/~latte}}

\item{quiet}{Show latte output}

\item{shell}{Messages the shell code used to do the computation}
}
\value{
A named list with components \code{par}, a named-vector of optimizing
arguments, and \code{value}, the value of the objective function at the
optimial point.
}
\description{
\code{latte_max} and \code{latte_min} use LattE's \code{latte-maximize} and
\code{latte-minimize} functions to find the maximum or minimum of a linear
objective function over the integers points in a polytope (i.e. satisfying
linearity constraints). This makes use of the digging algorithm; see the
LattE manual at \url{http://www.math.ucdavis.edu/~latte} for details.
}
\examples{


if (has_latte()) {

latte_max(
  "-2 x + 3 y", 
  c("x + y <= 10", "x >= 0", "y >= 0")
)

latte_max(
  "-2 x + 3 y", 
  c("x + y <= 10", "x >= 0", "y >= 0"),
  quiet = FALSE
)


df <- expand.grid("x" = 0:10, "y" = 0:10)
df <- subset(df, x + y <= 10L)
df$objective <- with(df, -2*x + 3*y)
library("ggplot2")
ggplot(df, aes(x, y, size = objective)) +
  geom_point()

latte_min(
  "-2 x + 3 y",
  c("x + y <= 10", "x >= 0", "y >= 0"),
  method = "cones"
)



latte_min("-2 x - 3 y - 4 z", c(
  "3 x + 2 y + z <= 10",
  "2 x + 5 y + 3 z <= 15",
  "x >= 0", "y >= 0", "z >= 0"
), "cones", quiet = FALSE)





}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tab2vec.R
\name{tab2vec}
\alias{tab2vec}
\title{Array to vector conversion}
\usage{
tab2vec(tab)
}
\arguments{
\item{tab}{An array of counts}
}
\value{
a Named integer vector.  The names correspond to the cell indices in
the table.
}
\description{
Convert an array into a vector.
}
\details{
This function converts an array (or a multi-way contingency table) into a
vector, using a consistent ordering of the cells. The ordering of the cells
is lexicographical and cannot be specified by the user.
}
\examples{

a <- array(1:24, c(2,3,4))
tab2vec(a)

data(Titanic)
tab2vec(Titanic)
Titanic[1,1,1,1]
Titanic[1,1,1,2]


}
\seealso{
\code{\link[=vec2tab]{vec2tab()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vec2tab.R
\name{vec2tab}
\alias{vec2tab}
\title{Vector to array conversion}
\usage{
vec2tab(vec, dim)
}
\arguments{
\item{vec}{A vector}

\item{dim}{The desired array dimensions, oftentimes a vector of the number of
levels of each variable in order}
}
\value{
An array
}
\description{
Convert a vector into an array given a set of dimensions; it therefore simply
wraps \code{\link[=aperm]{aperm()}} and \code{\link[=array]{array()}}.
}
\details{
This function converts an array (or a multi-way contingency table) into a
vector, using a consistent ordering of the cells. The ordering of the cells
is lexicographical and cannot be specified by the user.
}
\examples{

data(Titanic)
Titanic
tab2vec(Titanic)
vec2tab(tab2vec(Titanic), dim(Titanic))
vec2tab(tab2vec(Titanic), dim(Titanic)) == Titanic
all(vec2tab(tab2vec(Titanic), dim(Titanic)) == Titanic)


}
\seealso{
\code{\link[=tab2vec]{tab2vec()}}, \code{\link[=aperm]{aperm()}}, \code{\link[=array]{array()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zsolve.R
\name{zsolve}
\alias{zsolve}
\title{Solve a linear system over the integers}
\usage{
zsolve(mat, rel, rhs, sign, lat, lb, ub, dir = tempdir(), quiet = TRUE,
  shell = FALSE, ...)
}
\arguments{
\item{mat}{The A matrix (see the 4ti2 documentation or examples)}

\item{rel}{A vector of "<" or ">" relations}

\item{rhs}{The right hand side b}

\item{sign}{The signs of the individual}

\item{lat}{A lattice basis (instead of a matrix)}

\item{lb}{Lower bounds on columns}

\item{ub}{Upper bounds on columns}

\item{dir}{Directory to place the files in, without an ending /}

\item{quiet}{If FALSE, messages the 4ti2 output}

\item{shell}{Messages the shell code used to do the computation}

\item{...}{Additional arguments to pass to the function}
}
\value{
The configuration matrix of the model provided
}
\description{
zsolve runs 4ti2's zsolve program to compute the configuration matrix A
corresponding to graphical statistical models given by a simplicial complex
and levels on the nodes.
}
\examples{

if (has_4ti2()) {

mat <- rbind(
  c( 1, -1),
  c(-3,  1),
  c( 1,  1)
)
rel <- c("<", "<", ">")
rhs <- c(2, 1, 1)
sign <- c(0, 1)

zsolve(mat, rel, rhs, sign)
zsolve(mat, rel, rhs, sign, quiet = FALSE)
zsolve(mat, rel, rhs, sign, shell = TRUE)

zsolve(mat, rel, rhs, sign, p = "gmp", quiet = FALSE)

}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsolve.R
\name{qsolve}
\alias{qsolve}
\title{Solve a linear system over the rationals}
\usage{
qsolve(mat, rel, sign, dir = tempdir(), quiet = TRUE, shell = FALSE,
  ...)
}
\arguments{
\item{mat}{The A matrix (see the 4ti2 documentation or examples)}

\item{rel}{A vector of "<" or ">" relations}

\item{sign}{The signs of the individual}

\item{dir}{Directory to place the files in, without an ending /}

\item{quiet}{If FALSE, messages the 4ti2 output}

\item{shell}{Messages the shell code used to do the computation}

\item{...}{Additional arguments to pass to the function}
}
\value{
The configuration matrix of the model provided
}
\description{
qsolve runs 4ti2's qsolve program to compute the
configuration matrix A corresponding to graphical statistical
models given by a simplicial complex and levels on the nodes.
}
\examples{

if (has_4ti2()) {

# x + y > 0
# x + y < 0

mat <- rbind(
  c( 1,  1),
  c( 1,  1)
)
rel <- c(">", "<")
sign <- c(0, 0)

qsolve(mat, rel, sign, p = "arb")
qsolve(mat, rel, sign, p = "arb", quiet = FALSE)
qsolve(mat, rel, sign, p = "arb", shell = TRUE)

}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genmodel.R
\name{genmodel}
\alias{genmodel}
\title{Generate a configuration matrix}
\usage{
genmodel(varlvls, facets, dir = tempdir(), quiet = TRUE,
  shell = FALSE, ...)
}
\arguments{
\item{varlvls}{a vector containing the number of levels of each variable}

\item{facets}{the facets generating the hierarchical model, a list of vectors
of variable indices}

\item{dir}{Directory to place the files in, without an ending /}

\item{quiet}{If FALSE, messages the 4ti2 output}

\item{shell}{Messages the shell code used to do the computation}

\item{...}{Additional arguments to pass to the function}
}
\value{
The configuration matrix of the model provided
}
\description{
genmodel runs 4ti2's genmodel program to compute the configuration matrix A
corresponding to graphical statistical models given by a simplicial complex
and levels on the nodes.
}
\examples{

if (has_4ti2()) {

varlvls <- rep(2, 2)
facets <- list(1, 2)
genmodel(varlvls, facets)
genmodel(varlvls, facets, quiet = FALSE)

varlvls <- rep(3, 3)
facets <- list(1:2, 2:3, c(3,1))
genmodel(varlvls, facets)

# compare this to algstat's hmat function

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latte.R
\docType{package}
\name{latte}
\alias{latte}
\alias{package-latte}
\alias{latte-package}
\title{R Interface to LattE and 4ti2}
\description{
Back-end connections to LattE (\url{https://www.math.ucdavis.edu/~latte/})
and 4ti2 (\url{http://www.4ti2.de/}) executables and front-end tools
facilitating their use in the R ecosystem.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableau.R
\name{print.tableau}
\alias{print.tableau}
\title{Pretty printing of tableau output.}
\usage{
\method{print}{tableau}(x, ...)
}
\arguments{
\item{x}{an object of class tableau}

\item{...}{...}
}
\value{
Invisible string of the printed object.
}
\description{
Pretty printing of tableau output.
}
\examples{

# see ?tableau


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-matrix.R
\name{plot-matrix}
\alias{plot-matrix}
\alias{plot_matrix}
\title{Plot a matrix}
\usage{
plot_matrix(A)
}
\arguments{
\item{A}{A matrix}
}
\value{
a ggplot object
}
\description{
plot_matrix is a R variant of Matlab's \code{spy} function.
}
\examples{

# the no-three-way interaction configuration
(A <- kprod(ones(1,3), diag(3), ones(3)))
plot_matrix(A)


if (has_4ti2()) {

plot_matrix(markov(A))

(A <- genmodel(c(2L, 2L), list(1L, 2L)))
plot_matrix(A)
plot_matrix(markov(A))

(A <- genmodel(c(5L, 5L), list(1L, 2L)))
plot_matrix(A)
plot_matrix(markov(A))

}

}
\author{
David Kahle \email{david@kahle.io}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kprod.R
\name{kprod}
\alias{kprod}
\title{Iterated Kronecker product}
\usage{
kprod(..., FUN = `*`)
}
\arguments{
\item{...}{A listing of matrices}

\item{FUN}{A function to pass to \code{\link[=kronecker]{kronecker()}}}
}
\value{
A matrix that is the kronecker product of the specified matrices
(from left to right).
}
\description{
Compute the Kronecker product of several matrices.
}
\details{
If kronecker is  the function that computes A x B, kprod computes A x B x C
and so on; it's a wrapper of Reduce and kronecker.
}
\examples{

kprod(diag(2), t(ones(2)))
kprod(t(ones(2)), diag(2))


kprod(diag(2), t(ones(2)), t(ones(2)))
kprod(t(ones(2)), diag(2), t(ones(2)))
kprod(t(ones(2)), t(ones(2)), diag(2))


# cf. aoki, hara, and takemura p.13
rbind(
  kprod(diag(2), t(ones(2))),
  kprod(t(ones(2)), diag(2))
) 


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latte-files.R
\name{latte-files}
\alias{latte-files}
\alias{format_latte}
\alias{write_latte}
\alias{write.latte}
\alias{read_latte}
\alias{read.latte}
\title{Format/read/write a matrix in latte's style}
\usage{
format_latte(mat, file)

write_latte(mat, file)

write.latte(mat, file)

read_latte(file, format = c("mat", "Ab"))

read.latte(file, format = c("mat", "Ab"))
}
\arguments{
\item{mat}{A matrix}

\item{file}{A filename}

\item{format}{"mat" or "Ab"}
}
\value{
\itemize{

\item \code{\link[=format_latte]{format_latte()}} -- A character string of the matrix in
latte format.

\item \code{\link[=write_latte]{write_latte()}} -- An invisible character
string of the formatted output.

\item \code{\link[=read_latte]{read_latte()}} -- An integer matrix.

}
}
\description{
\code{\link[=format_latte]{format_latte()}} formats a matrix in latte's style.
\code{\link[=write_latte]{write_latte()}} writes a latte-formatted file to file.
\code{\link[=read_latte]{read_latte()}} reads a latte-formatted file from disk.
}
\examples{


(mat <- matrix(sample(9), 3, 3))

format_latte(mat)
cat(format_latte(mat))

(file <- file.path(tempdir(), "foo.hrep"))
write_latte(mat, file)
file.show(file)
read_latte(file)
read_latte(file, "Ab")

attr(mat, "linearity") <- c(1, 3)
attr(mat, "nonnegative") <- 2
mat
format_latte(mat)
cat(format_latte(mat))
write_latte(mat, file)
file.show(file)
read_latte(file)

file.remove(file)



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/call-latte.R
\name{call_latte}
\alias{call_latte}
\title{Unified interface to calling LattE}
\usage{
call_latte(
  which_exe,
  input_file,
  project_dir,
  opts = list(),
  stdout = "stdout",
  stderr = "stderr",
  shell = FALSE
)
}
\arguments{
\item{which_exe}{The name of LattE executable, e.g. \code{"count"}}

\item{input_file}{The filename of file to be given to the executable, without
a path}

\item{project_dir}{The path associated with \code{input_file}.}

\item{opts}{A named list of optional command line arguments to the
executable.}

\item{stdout, stderr}{Filenames to redirect standard output and standard error
to.}

\item{shell}{\code{logical(1)}; should the shell command be messaged to the
user?}
}
\value{
\code{invisible()}
}
\description{
Unified interface to calling LattE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set-paths.R
\name{pathing}
\alias{pathing}
\alias{set_latte_path}
\alias{set_4ti2_path}
\alias{get_4ti2_path}
\alias{get_latte_path}
\alias{has_4ti2}
\alias{has_latte}
\alias{missing_4ti2_stop}
\alias{missing_latte_stop}
\title{Set paths to LattE and 4ti2 executables}
\usage{
set_latte_path(path)

set_4ti2_path(path)

get_4ti2_path()

get_latte_path()

has_4ti2()

has_latte()

missing_4ti2_stop()

missing_latte_stop()
}
\arguments{
\item{path}{A character string, the path to a 4ti2 function (e.g. \code{markov})
for setting 4ti2's path or a LattE function (e.g. \code{count}) for LattE's path}
}
\value{
A logical(1) or character(1) containing the path.
}
\description{
These are helper functions that deal with pathing to external programs and
asking if they are present. When latte is loaded it attempts to find LattE
and 4ti2 executables by looking for environment variables specifying them,
i.e. their paths as specified in your \code{.Renviron} file.
}
\details{
For easiest use, you'll want to specify the paths of LattE and 4ti2
executables in your \verb{~/.Renviron} file. They should look something like

\verb{LATTE=/Applications/latte/bin}

\verb{FOURTITWO=/Applications/latte/bin}

You can set these permanently with \code{\link[usethis:edit]{usethis::edit_r_environ()}}.

You can change these for the current session using, e.g., \code{\link[=set_latte_path]{set_latte_path()}},
which accepts a character string or, if missing, uses \code{\link[=file.choose]{file.choose()}} to let
you interactively; you just select an arbitrary executable.
}
\examples{


has_4ti2()
if (has_4ti2()) get_4ti2_path()

has_latte()
if (has_4ti2()) get_latte_path()


# you can set these paths permanently with the following. note that you'll 
# need to re-start the R session afterwards or simply pass the path into, 
# e.g., set_latte_path(). see below for more details on that.
if (interactive()) edit_r_environ()


# you can change these in your current session with set_latte_path() and
if (had_latte <- has_latte()) old_latte_path <- get_latte_path()
set_latte_path("/path/to/latte")
get_latte_path()

if (had_latte) set_latte_path(old_latte_path)
get_latte_path()

}
\author{
David Kahle \email{david@kahle.io}
}
